import logging
from typing import Optional

from dateutil.parser import parse
from django.utils import timezone
from music.constants import JELLYFIN_POST_KEYS
from music.models import Track
from podcasts.models import Episode
from scrobbles.models import Scrobble
from scrobbles.utils import convert_to_seconds, parse_mopidy_uri
from videos.models import Video
from sports.models import SportEvent

logger = logging.getLogger(__name__)


def mopidy_scrobble_podcast(
    data_dict: dict, user_id: Optional[int]
) -> Scrobble:
    mopidy_uri = data_dict.get("mopidy_uri", "")
    parsed_data = parse_mopidy_uri(mopidy_uri)

    producer_dict = {"name": data_dict.get("artist")}

    podcast_name = data_dict.get("album")
    if not podcast_name:
        podcast_name = parsed_data.get("podcast_name")
    podcast_dict = {"name": podcast_name}

    episode_name = parsed_data.get("episode_filename")
    episode_dict = {
        "title": episode_name,
        "run_time_ticks": data_dict.get("run_time_ticks"),
        "run_time": data_dict.get("run_time"),
        "number": parsed_data.get("episode_num"),
        "pub_date": parsed_data.get("pub_date"),
        "mopidy_uri": mopidy_uri,
    }

    episode = Episode.find_or_create(podcast_dict, producer_dict, episode_dict)

    # Now we run off a scrobble
    mopidy_data = {
        "user_id": user_id,
        "timestamp": timezone.now(),
        "playback_position_ticks": data_dict.get("playback_time_ticks"),
        "source": "Mopidy",
        "mopidy_status": data_dict.get("status"),
    }

    scrobble = None
    if episode:
        scrobble = Scrobble.create_or_update(episode, user_id, mopidy_data)
    return scrobble


def mopidy_scrobble_track(
    data_dict: dict, user_id: Optional[int]
) -> Optional[Scrobble]:
    artist_dict = {
        "name": data_dict.get("artist", None),
        "musicbrainz_id": data_dict.get("musicbrainz_artist_id", None),
    }

    album_dict = {
        "name": data_dict.get("album"),
        "musicbrainz_id": data_dict.get("musicbrainz_album_id"),
    }

    track_dict = {
        "title": data_dict.get("name"),
        "run_time_ticks": data_dict.get("run_time_ticks"),
        "run_time": data_dict.get("run_time"),
    }

    track = Track.find_or_create(artist_dict, album_dict, track_dict)

    # Now we run off a scrobble
    mopidy_data = {
        "user_id": user_id,
        "timestamp": timezone.now(),
        "playback_position_ticks": data_dict.get("playback_time_ticks"),
        "source": "Mopidy",
        "mopidy_status": data_dict.get("status"),
    }

    # Jellyfin MB ids suck, so always overwrite with Mopidy if they're offering
    track.musicbrainz_id = data_dict.get("musicbrainz_track_id")
    track.save()

    scrobble = Scrobble.create_or_update(track, user_id, mopidy_data)

    return scrobble


def build_scrobble_dict(data_dict: dict, user_id: int) -> dict:
    jellyfin_status = "resumed"
    if data_dict.get("IsPaused"):
        jellyfin_status = "paused"
    elif data_dict.get("NotificationType") == 'PlaybackStop':
        jellyfin_status = "stopped"

    playback_ticks = data_dict.get("PlaybackPositionTicks", "")
    if playback_ticks:
        playback_ticks = playback_ticks // 10000

    return {
        "user_id": user_id,
        "timestamp": parse(data_dict.get("UtcTimestamp")),
        "playback_position_ticks": playback_ticks,
        "playback_position": data_dict.get("PlaybackPosition", ""),
        "source": data_dict.get("ClientName", "Vrobbler"),
        "source_id": data_dict.get('MediaSourceId'),
        "jellyfin_status": jellyfin_status,
    }


def jellyfin_scrobble_track(
    data_dict: dict, user_id: Optional[int]
) -> Optional[Scrobble]:

    if not data_dict.get("Provider_musicbrainztrack", None):
        # TODO we should be able to look up tracks via MB rather than error out
        logger.error(
            "No MBrainz Track ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return

    null_position_on_progress = (
        data_dict.get("PlaybackPosition") == "00:00:00"
        and data_dict.get("NotificationType") == "PlaybackProgress"
    )

    # Jellyfin has some race conditions with it's webhooks, these hacks fix some of them
    if not data_dict.get("PlaybackPositionTicks") or null_position_on_progress:
        logger.error("No playback position tick from Jellyfin, aborting")
        return

    artist_dict = {
        'name': data_dict.get(JELLYFIN_POST_KEYS["ARTIST_NAME"], None),
        'musicbrainz_id': data_dict.get(
            JELLYFIN_POST_KEYS["ARTIST_MB_ID"], None
        ),
    }

    album_dict = {
        "name": data_dict.get(JELLYFIN_POST_KEYS["ALBUM_NAME"], None),
        "musicbrainz_id": data_dict.get(JELLYFIN_POST_KEYS['ALBUM_MB_ID']),
    }

    # Convert ticks from Jellyfin from microseconds to nanoseconds
    # Ain't nobody got time for nanoseconds
    track_dict = {
        "title": data_dict.get("Name", ""),
        "run_time_ticks": data_dict.get(
            JELLYFIN_POST_KEYS["RUN_TIME_TICKS"], None
        )
        // 10000,
        "run_time": convert_to_seconds(
            data_dict.get(JELLYFIN_POST_KEYS["RUN_TIME"], None)
        ),
    }
    track = Track.find_or_create(artist_dict, album_dict, track_dict)

    # Prefer Mopidy MD IDs to Jellyfin, so skip if we already have one
    if not track.musicbrainz_id:
        track.musicbrainz_id = data_dict.get(
            JELLYFIN_POST_KEYS["TRACK_MB_ID"], None
        )
        track.save()

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    # A hack to make Jellyfin work more like Mopidy for music tracks
    scrobble_dict["playback_position_ticks"] = 0
    scrobble_dict["playback_position"] = ""

    return Scrobble.create_or_update(track, user_id, scrobble_dict)


def jellyfin_scrobble_video(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_imdb", None):
        logger.error(
            "No IMDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    video = Video.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(video, user_id, scrobble_dict)


def manual_scrobble_video(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_imdb", None):
        logger.error(
            "No IMDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    video = Video.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(video, user_id, scrobble_dict)


def manual_scrobble_event(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_thesportsdb", None):
        logger.error(
            "No TheSportsDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    event = SportEvent.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(event, user_id, scrobble_dict)



# Generated by Django 4.1.5 on 2023-01-14 21:14

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='League',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
                (
                    'logo',
                    models.ImageField(
                        blank=True, null=True, upload_to='sports/league-logos/'
                    ),
                ),
                (
                    'abbreviation_str',
                    models.CharField(blank=True, max_length=10, null=True),
                ),
                ('thesportsdb_id', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
                ('thesportsdb_id', models.IntegerField(blank=True, null=True)),
                (
                    'league',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='sports.league',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SportEvent',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
                (
                    'title',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                (
                    'run_time',
                    models.CharField(blank=True, max_length=8, null=True),
                ),
                (
                    'run_time_ticks',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                (
                    'event_type',
                    models.CharField(
                        choices=[
                            ('UK', 'Unknown'),
                            ('GA', 'Game'),
                            ('MA', 'Match'),
                            ('ME', 'Meet'),
                        ],
                        default='UK',
                        max_length=2,
                    ),
                ),
                ('start_utc', models.DateTimeField(blank=True, null=True)),
                (
                    'season',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                (
                    'away_team',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name='away_event_set',
                        to='sports.team',
                    ),
                ),
                (
                    'home_team',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name='home_event_set',
                        to='sports.team',
                    ),
                ),
                (
                    'league',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='sports.league',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import calendar
import logging
from datetime import datetime, timedelta
from typing import Optional

import pytz
from django.apps import apps
from django.conf import settings
from django.db.models import Count, Q


logger = logging.getLogger(__name__)


def get_start_end_dates_by_week(year, week, tz):
    d = datetime(year, 1, 1, tzinfo=tz)
    if d.weekday() <= 3:
        d = d - timedelta(d.weekday())
    else:
        d = d + timedelta(7 - d.weekday())
    dlt = timedelta(days=(week - 1) * 7)
    return d + dlt, d + dlt + timedelta(days=6)


def get_scrobble_count_qs(
    year: Optional[int] = None,
    month: Optional[int] = None,
    week: Optional[int] = None,
    day: Optional[int] = None,
    user=None,
    model_str="Track",
) -> dict[str, int]:

    tz = settings.TIME_ZONE
    if user and user.is_authenticated:
        tz = pytz.timezone(user.profile.timezone)

    data_model = apps.get_model(app_label='music', model_name='Track')
    if model_str == "Video":
        data_model = apps.get_model(app_label='videos', model_name='Video')
    if model_str == "SportEvent":
        data_model = apps.get_model(
            app_label='sports', model_name='SportEvent'
        )

    base_qs = data_model.objects.filter(
        scrobble__user=user,
        scrobble__played_to_completion=True,
    )

    # Returna all media items with scrobble count annotated
    if not year:
        return base_qs.annotate(scrobble_count=Count("scrobble")).order_by(
            "-scrobble_count"
        )

    start = datetime(year, 1, 1, tzinfo=tz)
    end = datetime(year, 12, 31, tzinfo=tz)
    if month:
        end_day = calendar.monthrange(year, month)[1]
        start = datetime(year, month, 1, tzinfo=tz)
        end = datetime(year, month, end_day, tzinfo=tz)
    elif week:
        start, end = get_start_end_dates_by_week(year, week, tz)
    elif day and month:
        start = datetime(year, month, day, 0, 0, tzinfo=tz)
        end = datetime(year, month, day, 23, 59, tzinfo=tz)
    elif day and not month:
        logger.warning('Day provided with month, defaulting ot all-time')

    date_filter = Q(
        scrobble__timestamp__gte=start, scrobble__timestamp__lte=end
    )

    return (
        base_qs.annotate(
            scrobble_count=Count("scrobble", filter=Q(date_filter))
        )
        .filter(date_filter)
        .order_by("-scrobble_count")
    )


def build_charts(
    year: Optional[int] = None,
    month: Optional[int] = None,
    week: Optional[int] = None,
    day: Optional[int] = None,
    user=None,
    model_str="Track",
):
    ChartRecord = apps.get_model(
        app_label='scrobbles', model_name='ChartRecord'
    )
    results = get_scrobble_count_qs(year, month, week, day, user, model_str)
    unique_counts = list(set([result.scrobble_count for result in results]))
    unique_counts.sort(reverse=True)
    ranks = {}
    for rank, count in enumerate(unique_counts, start=1):
        ranks[count] = rank

    chart_records = []
    for result in results:
        chart_record = {
            'year': year,
            'week': week,
            'month': month,
            'day': day,
            'user': user,
        }
        chart_record['rank'] = ranks[result.scrobble_count]
        if model_str == 'Track':
            chart_record['track'] = result
        chart_records.append(ChartRecord(**chart_record))
    ChartRecord.objects.bulk_create(chart_records, ignore_conflicts=True)



from django.contrib import admin

from music.models import Artist, Album, Track

from scrobbles.admin import ScrobbleInline


@admin.register(Album)
class AlbumAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "year", "musicbrainz_id")
    list_filter = ("year",)
    ordering = ("name",)
    filter_horizontal = [
        'artists',
    ]


@admin.register(Artist)
class ArtistAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "musicbrainz_id")
    ordering = ("name",)


@admin.register(Track)
class TrackAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = (
        "title",
        "album",
        "artist",
        "run_time",
        "musicbrainz_id",
    )
    list_filter = ("album", "artist")
    ordering = ("-created",)
    inlines = [
        ScrobbleInline,
    ]



# Generated by Django 4.1.5 on 2023-01-05 17:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0003_remove_scrobble_counted_scrobble_in_progress'),
    ]

    operations = [
        migrations.AddField(
            model_name='scrobble',
            name='scrobble_log',
            field=models.TextField(blank=True, null=True),
        ),
    ]



import os
import sys
from pathlib import Path

import dj_database_url
from django.utils.translation import gettext_lazy as _
from dotenv import load_dotenv

PROJECT_ROOT = Path(__file__).resolve().parent
BASE_DIR = Path(__file__).resolve().parent.parent
sys.path.insert(0, os.path.join(PROJECT_ROOT, 'apps'))

# Tap vrobbler.conf if it's available
if os.path.exists("vrobbler.conf"):
    load_dotenv("vrobbler.conf")
elif os.path.exists("/etc/vrobbler.conf"):
    load_dotenv("/etc/vrobbler.conf")
elif os.path.exists("/usr/local/etc/vrobbler.conf"):
    load_dotenv("/usr/local/etc/vrobbler.conf")

# Build paths inside the project like this: BASE_DIR / 'subdir'.


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.getenv("VROBBLER_SECRET_KEY", "not-a-secret-234lkjasdflj132")


# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.getenv("VROBBLER_DEBUG", False)

TESTING = len(sys.argv) > 1 and sys.argv[1] == "test"

KEEP_DETAILED_SCROBBLE_LOGS = os.getenv(
    "VROBBLER_KEEP_DETAILED_SCROBBLE_LOGS", False
)


# Should we cull old in-progress scrobbles that are beyond the wait period for resuming?
DELETE_STALE_SCROBBLES = os.getenv("VROBBLER_DELETE_STALE_SCROBBLES", True)

# Used to dump data coming from srobbling sources, helpful for building new inputs
DUMP_REQUEST_DATA = os.getenv("VROBBLER_DUMP_REQUEST_DATA", False)


THESPORTSDB_API_KEY = os.getenv("VROBBLER_THESPORTSDB_API_KEY", "2")
THESPORTSDB_BASE_URL = os.getenv(
    "VROBBLER_THESPORTSDB_BASE_URL", "https://www.thesportsdb.com/api/v1/json/"
)
TMDB_API_KEY = os.getenv("VROBBLER_TMDB_API_KEY", "")

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

TIME_ZONE = os.getenv("VROBBLER_TIME_ZONE", "US/Eastern")

ALLOWED_HOSTS = ["*"]
CSRF_TRUSTED_ORIGINS = [
    os.getenv("VROBBLER_TRUSTED_ORIGINS", "http://localhost:8000")
]
X_FRAME_OPTIONS = "SAMEORIGIN"

CACHALOT_TIMEOUT = os.getenv("VROBBLER_CACHALOT_TIMEOUT", 3600)
REDIS_URL = os.getenv("VROBBLER_REDIS_URL", None)

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "django.contrib.humanize",
    "django_filters",
    "django_extensions",
    'rest_framework.authtoken',
    "cachalot",
    "profiles",
    "scrobbles",
    "videos",
    "music",
    "podcasts",
    "sports",
    "mathfilters",
    "rest_framework",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "django_celery_results",
]

SITE_ID = 1

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.gzip.GZipMiddleware",
]

ROOT_URLCONF = "vrobbler.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [str(PROJECT_ROOT.joinpath("templates"))],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "videos.context_processors.video_lists",
                "music.context_processors.music_lists",
            ],
        },
    },
]

WSGI_APPLICATION = "vrobbler.wsgi.application"

DATABASES = {
    "default": dj_database_url.config(
        default=os.getenv("VROBBLER_DATABASE_URL", "sqlite:///db.sqlite3"),
        conn_max_age=600,
    ),
}
if TESTING:
    DATABASES = {
        "default": dj_database_url.config(default="sqlite:///testdb.sqlite3")
    }


CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
    }
}
if REDIS_URL:
    CACHES["default"]["BACKEND"] = "django_redis.cache.RedisCache"
    CACHES["default"]["LOCATION"] = REDIS_URL

SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
]

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": ("rest_framework.permissions.AllowAny",),
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    ],
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "DEFAULT_FILTER_BACKENDS": [
        "django_filters.rest_framework.DjangoFilterBackend"
    ],
    'DEFAULT_PARSER_CLASSES': [
        'rest_framework.parsers.JSONParser',
    ],
    'DEFAULT_CONTENT_NEGOTIATION_CLASS': 'vrobbler.negotiation.IgnoreClientContentNegotiation',
    "PAGE_SIZE": 100,
}

LOGIN_REDIRECT_URL = "/"

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = os.getenv("VROBBLER_TIME_ZONE", "EST")

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = "static/"
STATIC_ROOT = os.getenv(
    "VROBBLER_STATIC_ROOT", os.path.join(PROJECT_ROOT, "static")
)

MEDIA_URL = "/media/"
MEDIA_ROOT = os.getenv(
    "VROBBLER_MEDIA_ROOT", os.path.join(PROJECT_ROOT, "media")
)

JSON_LOGGING = os.getenv("VROBBLER_JSON_LOGGING", False)
LOG_TYPE = "json" if JSON_LOGGING else "log"

default_level = "INFO"
if DEBUG:
    default_level = "DEBUG"

LOG_LEVEL = os.getenv("VROBBLER_LOG_LEVEL", default_level)
LOG_FILE_PATH = os.getenv("VROBBLER_LOG_FILE_PATH", "/tmp/")

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "root": {
        "handlers": ["console", "file"],
        "level": LOG_LEVEL,
        "propagate": True,
    },
    "formatters": {
        "color": {
            "()": "colorlog.ColoredFormatter",
            # \r returns caret to line beginning, in tests this eats the silly dot that removes
            # the beautiful alignment produced below
            "format": "\r"
            "{log_color}{levelname:8s}{reset} "
            "{bold_cyan}{name}{reset}:"
            "{fg_bold_red}{lineno}{reset} "
            "{thin_yellow}{funcName} "
            "{thin_white}{message}"
            "{reset}",
            "style": "{",
        },
        "log": {"format": "%(asctime)s %(levelname)s %(message)s"},
        "json": {
            "()": "pythonjsonlogger.jsonlogger.JsonFormatter",
            "format": "%(levelname)s %(name) %(funcName) %(lineno) %(asctime)s %(message)s",
        },
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "color",
            "level": LOG_LEVEL,
        },
        "null": {
            "class": "logging.NullHandler",
            "level": LOG_LEVEL,
        },
        'sql': {
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': ''.join([LOG_FILE_PATH, 'vrobbler_sql.', LOG_TYPE]),
            'formatter': LOG_TYPE,
            'level': LOG_LEVEL,
        },
        'file': {
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': ''.join([LOG_FILE_PATH, 'vrobbler.', LOG_TYPE]),
            'formatter': LOG_TYPE,
            'level': LOG_LEVEL,
        },
    },
    "loggers": {
        # Quiet down our console a little
        "django": {
            "handlers": ["file"],
            "propagate": True,
        },
        "django.db.backends": {"handlers": ["null"]},
        "django.server": {"handlers": ["null"]},
        "vrobbler": {
            "handlers": ["file"],
            "propagate": True,
        },
    },
}

LOG_TO_CONSOLE = os.getenv("VROBBLER_LOG_TO_CONSOLE", False)
if LOG_TO_CONSOLE:
    LOGGING['loggers']['django']['handlers'] = ["console"]
    LOGGING['loggers']['vrobbler']['handlers'] = ["console"]



from videos.models import Video, Series


def video_lists(request):
    return {
        "movie_list": Video.objects.filter(video_type=Video.VideoType.MOVIE),
        "series_list": Series.objects.all(),
    }



import scrobbles.views as scrobbles_views
from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include, path
from scrobbles import urls as scrobble_urls
from music import urls as music_urls
from videos import urls as video_urls

urlpatterns = [
    path("admin/", admin.site.urls),
    path("accounts/", include("allauth.urls")),
    # path("api-auth/", include("rest_framework.urls")),
    # path("movies/", include(movies, namespace="movies")),
    # path("shows/", include(shows, namespace="shows")),
    path("api/v1/scrobbles/", include(scrobble_urls, namespace="scrobbles")),
    path(
        'manual/imdb/',
        scrobbles_views.ManualScrobbleView.as_view(),
        name='imdb-manual-scrobble',
    ),
    path(
        'manual/audioscrobbler/',
        scrobbles_views.AudioScrobblerImportCreateView.as_view(),
        name='audioscrobbler-file-upload',
    ),
    path("", include(music_urls, namespace="music")),
    path("", include(video_urls, namespace="videos")),
    path(
        "", scrobbles_views.RecentScrobbleList.as_view(), name="vrobbler-home"
    ),
]

if settings.DEBUG:
    urlpatterns += static(
        settings.MEDIA_URL, document_root=settings.MEDIA_ROOT
    )
    urlpatterns += static(
        settings.STATIC_URL, document_root=settings.STATIC_ROOT
    )



from django.contrib import admin

from scrobbles.models import AudioScrobblerTSVImport, Scrobble


class ScrobbleInline(admin.TabularInline):
    model = Scrobble
    extra = 0
    raw_id_fields = ('video', 'podcast_episode', 'track')
    exclude = ('source_id', 'scrobble_log')


@admin.register(AudioScrobblerTSVImport)
class AudioScrobblerTSVImportAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("uuid", "created", "process_count", "tsv_file")
    ordering = ("-created",)


@admin.register(Scrobble)
class ScrobbleAdmin(admin.ModelAdmin):
    date_hierarchy = "timestamp"
    list_display = (
        "timestamp",
        "media_name",
        "media_type",
        "playback_percent",
        "source",
        "in_progress",
        "is_paused",
        "played_to_completion",
    )
    raw_id_fields = ('video', 'podcast_episode', 'track', 'sport_event')
    list_filter = ("is_paused", "in_progress", "source", "track__artist")
    ordering = ("-timestamp",)

    def media_name(self, obj):
        if obj.video:
            return obj.video
        if obj.track:
            return obj.track
        if obj.podcast_episode:
            return obj.podcast_episode
        if obj.sport_event:
            return obj.sport_event

    def media_type(self, obj):
        if obj.video:
            return "Video"
        if obj.track:
            return "Track"
        if obj.podcast_episode:
            return "Podcast"
        if obj.sport_event:
            return "Sport Event"

    def playback_percent(self, obj):
        return obj.percent_played



import calendar
import logging
from uuid import uuid4

from django.contrib.auth import get_user_model
from django.db import models
from django.utils import timezone
from django_extensions.db.models import TimeStampedModel
from music.models import Artist, Track
from podcasts.models import Episode
from scrobbles.utils import check_scrobble_for_finish
from sports.models import SportEvent
from videos.models import Series, Video
from vrobbler.apps.profiles.utils import now_user_timezone

logger = logging.getLogger(__name__)
User = get_user_model()
BNULL = {"blank": True, "null": True}


class AudioScrobblerTSVImport(TimeStampedModel):
    def get_path(instance, filename):
        extension = filename.split('.')[-1]
        uuid = instance.uuid
        return f'audioscrobbler-uploads/{uuid}.{extension}'

    user = models.ForeignKey(User, on_delete=models.DO_NOTHING, **BNULL)
    uuid = models.UUIDField(editable=False, default=uuid4)
    tsv_file = models.FileField(upload_to=get_path, **BNULL)
    processed_on = models.DateTimeField(**BNULL)
    process_log = models.TextField(**BNULL)
    process_count = models.IntegerField(**BNULL)

    def __str__(self):
        if self.tsv_file:
            return f"Audioscrobbler TSV upload: {self.tsv_file.path}"
        return f"Audioscrobbler TSV upload {self.id}"

    def save(self, **kwargs):
        """On save, attempt to import the TSV file"""
        super().save(**kwargs)
        self.process()
        return

    def process(self, force=False):
        from scrobbles.tsv import process_audioscrobbler_tsv_file

        if self.processed_on and not force:
            logger.info(f"{self} already processed on {self.processed_on}")
            return

        tz = None
        if self.user:
            tz = self.user.profile.tzinfo
        scrobbles = process_audioscrobbler_tsv_file(
            self.tsv_file.path, user_tz=tz
        )
        if scrobbles:
            self.process_log = f"Created {len(scrobbles)} scrobbles"
            for scrobble in scrobbles:
                scrobble_str = f"{scrobble.id}\t{scrobble.timestamp}\t{scrobble.track.title}"
                self.process_log += f"\n{scrobble_str}"
            self.process_count = len(scrobbles)
        else:
            self.process_log = f"Created no new scrobbles"
            self.process_count = 0

        self.processed_on = timezone.now()
        self.save(
            update_fields=['processed_on', 'process_count', 'process_log']
        )

    def undo(self, dryrun=True):
        from scrobbles.tsv import undo_audioscrobbler_tsv_import

        undo_audioscrobbler_tsv_import(self.process_log, dryrun)


class ChartRecord(TimeStampedModel):
    """Sort of like a materialized view for what we could dynamically generate,
    but would kill the DB as it gets larger. Collects time-based records
    generated by a cron-like archival job

    1972 by Josh Rouse - #3 in 2023, January

    """

    user = models.ForeignKey(User, on_delete=models.DO_NOTHING, **BNULL)
    rank = models.IntegerField()
    year = models.IntegerField(default=timezone.now().year)
    month = models.IntegerField(**BNULL)
    week = models.IntegerField(**BNULL)
    day = models.IntegerField(**BNULL)
    video = models.ForeignKey(Video, on_delete=models.DO_NOTHING, **BNULL)
    series = models.ForeignKey(Series, on_delete=models.DO_NOTHING, **BNULL)
    artist = models.ForeignKey(Artist, on_delete=models.DO_NOTHING, **BNULL)
    track = models.ForeignKey(Track, on_delete=models.DO_NOTHING, **BNULL)

    @property
    def media_obj(self):
        media_obj = None
        if self.video:
            media_obj = self.video
        if self.track:
            media_obj = self.track
        return media_obj

    @property
    def month_str(self) -> str:
        month_str = ""
        if self.month:
            month_str = calendar.month_name[self.month]
        return month_str

    @property
    def day_str(self) -> str:
        day_str = ""
        if self.day:
            day_str = str(self.day)
        return day_str

    @property
    def week_str(self) -> str:
        week_str = ""
        if self.week:
            week_str = str(self.week)
        return "Week " + week_str

    @property
    def period(self) -> str:
        period = str(self.year)
        if self.month:
            period = " ".join([self.month_str, period])
        if self.week:
            period = " ".join([self.week_str, period])
        if self.day:
            period = " ".join([self.day_str, period])
        return period

    @property
    def period_type(self) -> str:
        period = 'year'
        if self.month:
            period = 'month'
        if self.week:
            period = 'week'
        if self.day:
            period = 'day'
        return period

    def __str__(self):
        return f"#{self.rank} in {self.period} - {self.media_obj}"


class Scrobble(TimeStampedModel):
    """A scrobble tracks played media items by a user."""

    uuid = models.UUIDField(editable=False, **BNULL)
    video = models.ForeignKey(Video, on_delete=models.DO_NOTHING, **BNULL)
    track = models.ForeignKey(Track, on_delete=models.DO_NOTHING, **BNULL)
    podcast_episode = models.ForeignKey(
        Episode, on_delete=models.DO_NOTHING, **BNULL
    )
    sport_event = models.ForeignKey(
        SportEvent, on_delete=models.DO_NOTHING, **BNULL
    )
    user = models.ForeignKey(
        User, blank=True, null=True, on_delete=models.DO_NOTHING
    )
    timestamp = models.DateTimeField(**BNULL)
    playback_position_ticks = models.PositiveBigIntegerField(**BNULL)
    playback_position = models.CharField(max_length=8, **BNULL)
    is_paused = models.BooleanField(default=False)
    played_to_completion = models.BooleanField(default=False)
    source = models.CharField(max_length=255, **BNULL)
    source_id = models.TextField(**BNULL)
    in_progress = models.BooleanField(default=True)
    scrobble_log = models.TextField(**BNULL)

    def save(self, *args, **kwargs):
        if not self.uuid:
            self.uuid = uuid4()

        return super(Scrobble, self).save(*args, **kwargs)

    @property
    def status(self) -> str:
        if self.is_paused:
            return 'paused'
        if self.played_to_completion:
            return 'finished'
        if self.in_progress:
            return 'in-progress'
        return 'zombie'

    @property
    def is_stale(self) -> bool:
        """Mark scrobble as stale if it's been more than an hour since it was updated"""
        is_stale = False
        now = timezone.now()
        seconds_since_last_update = (now - self.modified).seconds
        if seconds_since_last_update >= self.media_obj.SECONDS_TO_STALE:
            is_stale = True
        return is_stale

    @property
    def percent_played(self) -> int:
        if not self.media_obj.run_time_ticks:
            return 100

        if not self.playback_position_ticks and self.played_to_completion:
            return 100

        playback_ticks = self.playback_position_ticks
        if not playback_ticks:
            playback_ticks = (timezone.now() - self.timestamp).seconds * 1000

        percent = int((playback_ticks / self.media_obj.run_time_ticks) * 100)
        if percent > 100:
            percent = 100
        return percent

    @property
    def can_be_updated(self) -> bool:
        updatable = True
        if self.percent_played > 100:
            logger.info(f"No - 100% played - {self.id} - {self.source}")
            updatable = False
        if self.is_stale:
            logger.info(f"No - stale - {self.id} - {self.source}")
            updatable = False
        return updatable

    @property
    def media_obj(self):
        media_obj = None
        if self.video:
            media_obj = self.video
        if self.track:
            media_obj = self.track
        if self.podcast_episode:
            media_obj = self.podcast_episode
        if self.sport_event:
            media_obj = self.sport_event
        return media_obj

    def __str__(self):
        timestamp = self.timestamp.strftime('%Y-%m-%d')
        return f"Scrobble of {self.media_obj} ({timestamp})"

    @classmethod
    def create_or_update(
        cls, media, user_id: int, scrobble_data: dict
    ) -> "Scrobble":

        if media.__class__.__name__ == 'Track':
            media_query = models.Q(track=media)
            scrobble_data['track_id'] = media.id
        if media.__class__.__name__ == 'Video':
            media_query = models.Q(video=media)
            scrobble_data['video_id'] = media.id
        if media.__class__.__name__ == 'Episode':
            media_query = models.Q(podcast_episode=media)
            scrobble_data['podcast_episode_id'] = media.id
        if media.__class__.__name__ == 'SportEvent':
            media_query = models.Q(sport_event=media)
            scrobble_data['sport_event_id'] = media.id

        scrobble = (
            cls.objects.filter(
                media_query,
                user_id=user_id,
            )
            .order_by('-modified')
            .first()
        )
        if scrobble and scrobble.can_be_updated:
            logger.info(
                f"Updating {scrobble.id}",
                {"scrobble_data": scrobble_data, "media": media},
            )
            return scrobble.update(scrobble_data)

        source = scrobble_data['source']
        logger.info(
            f"Creating for {media.id} - {source}",
            {"scrobble_data": scrobble_data, "media": media},
        )
        # If creating a new scrobble, we don't need status
        scrobble_data.pop('mopidy_status', None)
        scrobble_data.pop('jellyfin_status', None)
        return cls.create(scrobble_data)

    def update(self, scrobble_data: dict) -> "Scrobble":
        # Status is a field we get from Mopidy, which refuses to poll us
        scrobble_status = scrobble_data.pop('mopidy_status', None)
        if not scrobble_status:
            scrobble_status = scrobble_data.pop('jellyfin_status', None)

        if self.percent_played < 100:
            # Only worry about ticks if we haven't gotten to the end
            self.update_ticks(scrobble_data)

        # On stop, stop progress and send it to the check for completion
        if scrobble_status == "stopped":
            self.stop()
        # On pause, set is_paused and stop scrobbling
        if scrobble_status == "paused":
            self.pause()
        if scrobble_status == "resumed":
            self.resume()

        for key, value in scrobble_data.items():
            setattr(self, key, value)
        self.save()
        return self

    @classmethod
    def create(
        cls,
        scrobble_data: dict,
    ) -> "Scrobble":
        scrobble_data['scrobble_log'] = ""
        scrobble = cls.objects.create(
            **scrobble_data,
        )
        return scrobble

    def stop(self, force_finish=False) -> None:
        if not self.in_progress:
            return
        self.in_progress = False
        self.save(update_fields=['in_progress'])
        logger.info(f"{self.id} - {self.source}")
        check_scrobble_for_finish(self, force_finish)

    def pause(self) -> None:
        if self.is_paused:
            logger.warning(f"{self.id} - already paused - {self.source}")
            return
        self.is_paused = True
        self.save(update_fields=["is_paused"])
        logger.info(f"{self.id} - pausing - {self.source}")
        check_scrobble_for_finish(self)

    def resume(self) -> None:
        if self.is_paused or not self.in_progress:
            self.is_paused = False
            self.in_progress = True
            logger.info(f"{self.id} - resuming - {self.source}")
            return self.save(update_fields=["is_paused", "in_progress"])

    def cancel(self) -> None:
        check_scrobble_for_finish(self, force_finish=True)
        self.delete()

    def update_ticks(self, data) -> None:
        self.playback_position_ticks = data.get("playback_position_ticks")
        self.playback_position = data.get("playback_position")
        logger.info(
            f"{self.id} - {self.playback_position_ticks} - {self.source}"
        )
        self.save(
            update_fields=['playback_position_ticks', 'playback_position']
        )



from django.urls import path
from scrobbles import views

app_name = 'scrobbles'

urlpatterns = [
    path('', views.scrobble_endpoint, name='api-list'),
    path('finish/<slug:uuid>', views.scrobble_finish, name='finish'),
    path('cancel/<slug:uuid>', views.scrobble_cancel, name='cancel'),
    path(
        'upload/',
        views.AudioScrobblerImportCreateView.as_view(),
        name='audioscrobbler-file-upload',
    ),
    path('jellyfin/', views.jellyfin_websocket, name='jellyfin-websocket'),
    path('mopidy/', views.mopidy_websocket, name='mopidy-websocket'),
    path('export/', views.export, name='export'),
]



from django.contrib import admin

from sports.models import (
    League,
    Player,
    Round,
    Season,
    Sport,
    SportEvent,
    Team,
)

from scrobbles.admin import ScrobbleInline


@admin.register(Sport)
class SportAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    ordering = ("name",)


@admin.register(League)
class LeagueAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "abbreviation_str")
    ordering = ("name",)


@admin.register(Player)
class PlayerAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "league", "team")
    ordering = ("name",)


@admin.register(Season)
class SeasonAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "league")
    ordering = ("name",)


@admin.register(Round)
class RoundAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "season")
    ordering = ("name",)


@admin.register(Team)
class TeamAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "league")
    ordering = ("name",)


@admin.register(SportEvent)
class SportEventAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = (
        "title",
        "event_type",
        "start",
        "comp_str",
        "round",
    )
    list_filter = ("round__season", "home_team", "away_team")
    ordering = ("-created",)
    inlines = [
        ScrobbleInline,
    ]

    def comp_str(self, obj):
        if obj.home_team:
            return f'{obj.away_team} @ {obj.home_team}'
        if obj.player_one:
            return f'{obj.player_one} v {obj.player_two}'



from django.views import generic
from videos.models import Series, Video

# class VideoIndexView():


class MovieListView(generic.ListView):
    model = Video

    def get_queryset(self):
        return Video.objects.filter(video_type=Video.VideoType.MOVIE)


class SeriesListView(generic.ListView):
    model = Series


class SeriesDetailView(generic.DetailView):
    model = Series
    slug_field = 'uuid'


class VideoDetailView(generic.DetailView):
    model = Video
    slug_field = 'uuid'



from django.contrib.auth import get_user_model
from django.db.models.base import post_save
from django.dispatch import receiver

from profiles.models import UserProfile

User = get_user_model()


@receiver(post_save, sender=User)
def create_profile(sender, instance, created, **kwargs):
    if created:
        UserProfile.objects.create(user=instance)



# Generated by Django 4.1.5 on 2023-01-12 05:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music', '0005_album_cover_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='album',
            name='artists',
            field=models.ManyToManyField(
                blank=True, null=True, to='music.artist'
            ),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-18 03:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                (
                    'timezone',
                    models.CharField(
                        choices=[
                            ('Pacific/Midway', '(GMT-1100) Pacific/Midway'),
                            ('Pacific/Niue', '(GMT-1100) Pacific/Niue'),
                            (
                                'Pacific/Pago_Pago',
                                '(GMT-1100) Pacific/Pago_Pago',
                            ),
                            ('America/Adak', '(GMT-1000) America/Adak'),
                            (
                                'Pacific/Honolulu',
                                '(GMT-1000) Pacific/Honolulu',
                            ),
                            (
                                'Pacific/Rarotonga',
                                '(GMT-1000) Pacific/Rarotonga',
                            ),
                            ('Pacific/Tahiti', '(GMT-1000) Pacific/Tahiti'),
                            ('US/Hawaii', '(GMT-1000) US/Hawaii'),
                            (
                                'Pacific/Marquesas',
                                '(GMT-0930) Pacific/Marquesas',
                            ),
                            (
                                'America/Anchorage',
                                '(GMT-0900) America/Anchorage',
                            ),
                            ('America/Juneau', '(GMT-0900) America/Juneau'),
                            (
                                'America/Metlakatla',
                                '(GMT-0900) America/Metlakatla',
                            ),
                            ('America/Nome', '(GMT-0900) America/Nome'),
                            ('America/Sitka', '(GMT-0900) America/Sitka'),
                            ('America/Yakutat', '(GMT-0900) America/Yakutat'),
                            ('Pacific/Gambier', '(GMT-0900) Pacific/Gambier'),
                            ('US/Alaska', '(GMT-0900) US/Alaska'),
                            (
                                'America/Los_Angeles',
                                '(GMT-0800) America/Los_Angeles',
                            ),
                            ('America/Tijuana', '(GMT-0800) America/Tijuana'),
                            (
                                'America/Vancouver',
                                '(GMT-0800) America/Vancouver',
                            ),
                            ('Canada/Pacific', '(GMT-0800) Canada/Pacific'),
                            (
                                'Pacific/Pitcairn',
                                '(GMT-0800) Pacific/Pitcairn',
                            ),
                            ('US/Pacific', '(GMT-0800) US/Pacific'),
                            ('America/Boise', '(GMT-0700) America/Boise'),
                            (
                                'America/Cambridge_Bay',
                                '(GMT-0700) America/Cambridge_Bay',
                            ),
                            (
                                'America/Ciudad_Juarez',
                                '(GMT-0700) America/Ciudad_Juarez',
                            ),
                            ('America/Creston', '(GMT-0700) America/Creston'),
                            ('America/Dawson', '(GMT-0700) America/Dawson'),
                            (
                                'America/Dawson_Creek',
                                '(GMT-0700) America/Dawson_Creek',
                            ),
                            ('America/Denver', '(GMT-0700) America/Denver'),
                            (
                                'America/Edmonton',
                                '(GMT-0700) America/Edmonton',
                            ),
                            (
                                'America/Fort_Nelson',
                                '(GMT-0700) America/Fort_Nelson',
                            ),
                            (
                                'America/Hermosillo',
                                '(GMT-0700) America/Hermosillo',
                            ),
                            ('America/Inuvik', '(GMT-0700) America/Inuvik'),
                            (
                                'America/Mazatlan',
                                '(GMT-0700) America/Mazatlan',
                            ),
                            ('America/Phoenix', '(GMT-0700) America/Phoenix'),
                            (
                                'America/Whitehorse',
                                '(GMT-0700) America/Whitehorse',
                            ),
                            (
                                'America/Yellowknife',
                                '(GMT-0700) America/Yellowknife',
                            ),
                            ('Canada/Mountain', '(GMT-0700) Canada/Mountain'),
                            ('US/Arizona', '(GMT-0700) US/Arizona'),
                            ('US/Mountain', '(GMT-0700) US/Mountain'),
                            (
                                'America/Bahia_Banderas',
                                '(GMT-0600) America/Bahia_Banderas',
                            ),
                            ('America/Belize', '(GMT-0600) America/Belize'),
                            ('America/Chicago', '(GMT-0600) America/Chicago'),
                            (
                                'America/Chihuahua',
                                '(GMT-0600) America/Chihuahua',
                            ),
                            (
                                'America/Costa_Rica',
                                '(GMT-0600) America/Costa_Rica',
                            ),
                            (
                                'America/El_Salvador',
                                '(GMT-0600) America/El_Salvador',
                            ),
                            (
                                'America/Guatemala',
                                '(GMT-0600) America/Guatemala',
                            ),
                            (
                                'America/Indiana/Knox',
                                '(GMT-0600) America/Indiana/Knox',
                            ),
                            (
                                'America/Indiana/Tell_City',
                                '(GMT-0600) America/Indiana/Tell_City',
                            ),
                            ('America/Managua', '(GMT-0600) America/Managua'),
                            (
                                'America/Matamoros',
                                '(GMT-0600) America/Matamoros',
                            ),
                            (
                                'America/Menominee',
                                '(GMT-0600) America/Menominee',
                            ),
                            ('America/Merida', '(GMT-0600) America/Merida'),
                            (
                                'America/Mexico_City',
                                '(GMT-0600) America/Mexico_City',
                            ),
                            (
                                'America/Monterrey',
                                '(GMT-0600) America/Monterrey',
                            ),
                            (
                                'America/North_Dakota/Beulah',
                                '(GMT-0600) America/North_Dakota/Beulah',
                            ),
                            (
                                'America/North_Dakota/Center',
                                '(GMT-0600) America/North_Dakota/Center',
                            ),
                            (
                                'America/North_Dakota/New_Salem',
                                '(GMT-0600) America/North_Dakota/New_Salem',
                            ),
                            ('America/Ojinaga', '(GMT-0600) America/Ojinaga'),
                            (
                                'America/Rankin_Inlet',
                                '(GMT-0600) America/Rankin_Inlet',
                            ),
                            ('America/Regina', '(GMT-0600) America/Regina'),
                            (
                                'America/Resolute',
                                '(GMT-0600) America/Resolute',
                            ),
                            (
                                'America/Swift_Current',
                                '(GMT-0600) America/Swift_Current',
                            ),
                            (
                                'America/Tegucigalpa',
                                '(GMT-0600) America/Tegucigalpa',
                            ),
                            (
                                'America/Winnipeg',
                                '(GMT-0600) America/Winnipeg',
                            ),
                            ('Canada/Central', '(GMT-0600) Canada/Central'),
                            (
                                'Pacific/Galapagos',
                                '(GMT-0600) Pacific/Galapagos',
                            ),
                            ('US/Central', '(GMT-0600) US/Central'),
                            (
                                'America/Atikokan',
                                '(GMT-0500) America/Atikokan',
                            ),
                            ('America/Bogota', '(GMT-0500) America/Bogota'),
                            ('America/Cancun', '(GMT-0500) America/Cancun'),
                            ('America/Cayman', '(GMT-0500) America/Cayman'),
                            ('America/Detroit', '(GMT-0500) America/Detroit'),
                            (
                                'America/Eirunepe',
                                '(GMT-0500) America/Eirunepe',
                            ),
                            (
                                'America/Grand_Turk',
                                '(GMT-0500) America/Grand_Turk',
                            ),
                            (
                                'America/Guayaquil',
                                '(GMT-0500) America/Guayaquil',
                            ),
                            ('America/Havana', '(GMT-0500) America/Havana'),
                            (
                                'America/Indiana/Indianapolis',
                                '(GMT-0500) America/Indiana/Indianapolis',
                            ),
                            (
                                'America/Indiana/Marengo',
                                '(GMT-0500) America/Indiana/Marengo',
                            ),
                            (
                                'America/Indiana/Petersburg',
                                '(GMT-0500) America/Indiana/Petersburg',
                            ),
                            (
                                'America/Indiana/Vevay',
                                '(GMT-0500) America/Indiana/Vevay',
                            ),
                            (
                                'America/Indiana/Vincennes',
                                '(GMT-0500) America/Indiana/Vincennes',
                            ),
                            (
                                'America/Indiana/Winamac',
                                '(GMT-0500) America/Indiana/Winamac',
                            ),
                            ('America/Iqaluit', '(GMT-0500) America/Iqaluit'),
                            ('America/Jamaica', '(GMT-0500) America/Jamaica'),
                            (
                                'America/Kentucky/Louisville',
                                '(GMT-0500) America/Kentucky/Louisville',
                            ),
                            (
                                'America/Kentucky/Monticello',
                                '(GMT-0500) America/Kentucky/Monticello',
                            ),
                            ('America/Lima', '(GMT-0500) America/Lima'),
                            ('America/Nassau', '(GMT-0500) America/Nassau'),
                            (
                                'America/New_York',
                                '(GMT-0500) America/New_York',
                            ),
                            ('America/Panama', '(GMT-0500) America/Panama'),
                            (
                                'America/Port-au-Prince',
                                '(GMT-0500) America/Port-au-Prince',
                            ),
                            (
                                'America/Rio_Branco',
                                '(GMT-0500) America/Rio_Branco',
                            ),
                            ('America/Toronto', '(GMT-0500) America/Toronto'),
                            ('Canada/Eastern', '(GMT-0500) Canada/Eastern'),
                            ('Pacific/Easter', '(GMT-0500) Pacific/Easter'),
                            ('US/Eastern', '(GMT-0500) US/Eastern'),
                            (
                                'America/Anguilla',
                                '(GMT-0400) America/Anguilla',
                            ),
                            ('America/Antigua', '(GMT-0400) America/Antigua'),
                            ('America/Aruba', '(GMT-0400) America/Aruba'),
                            (
                                'America/Barbados',
                                '(GMT-0400) America/Barbados',
                            ),
                            (
                                'America/Blanc-Sablon',
                                '(GMT-0400) America/Blanc-Sablon',
                            ),
                            (
                                'America/Boa_Vista',
                                '(GMT-0400) America/Boa_Vista',
                            ),
                            (
                                'America/Campo_Grande',
                                '(GMT-0400) America/Campo_Grande',
                            ),
                            ('America/Caracas', '(GMT-0400) America/Caracas'),
                            ('America/Cuiaba', '(GMT-0400) America/Cuiaba'),
                            ('America/Curacao', '(GMT-0400) America/Curacao'),
                            (
                                'America/Dominica',
                                '(GMT-0400) America/Dominica',
                            ),
                            (
                                'America/Glace_Bay',
                                '(GMT-0400) America/Glace_Bay',
                            ),
                            (
                                'America/Goose_Bay',
                                '(GMT-0400) America/Goose_Bay',
                            ),
                            ('America/Grenada', '(GMT-0400) America/Grenada'),
                            (
                                'America/Guadeloupe',
                                '(GMT-0400) America/Guadeloupe',
                            ),
                            ('America/Guyana', '(GMT-0400) America/Guyana'),
                            ('America/Halifax', '(GMT-0400) America/Halifax'),
                            (
                                'America/Kralendijk',
                                '(GMT-0400) America/Kralendijk',
                            ),
                            ('America/La_Paz', '(GMT-0400) America/La_Paz'),
                            (
                                'America/Lower_Princes',
                                '(GMT-0400) America/Lower_Princes',
                            ),
                            ('America/Manaus', '(GMT-0400) America/Manaus'),
                            ('America/Marigot', '(GMT-0400) America/Marigot'),
                            (
                                'America/Martinique',
                                '(GMT-0400) America/Martinique',
                            ),
                            ('America/Moncton', '(GMT-0400) America/Moncton'),
                            (
                                'America/Montserrat',
                                '(GMT-0400) America/Montserrat',
                            ),
                            (
                                'America/Port_of_Spain',
                                '(GMT-0400) America/Port_of_Spain',
                            ),
                            (
                                'America/Porto_Velho',
                                '(GMT-0400) America/Porto_Velho',
                            ),
                            (
                                'America/Puerto_Rico',
                                '(GMT-0400) America/Puerto_Rico',
                            ),
                            (
                                'America/Santo_Domingo',
                                '(GMT-0400) America/Santo_Domingo',
                            ),
                            (
                                'America/St_Barthelemy',
                                '(GMT-0400) America/St_Barthelemy',
                            ),
                            (
                                'America/St_Kitts',
                                '(GMT-0400) America/St_Kitts',
                            ),
                            (
                                'America/St_Lucia',
                                '(GMT-0400) America/St_Lucia',
                            ),
                            (
                                'America/St_Thomas',
                                '(GMT-0400) America/St_Thomas',
                            ),
                            (
                                'America/St_Vincent',
                                '(GMT-0400) America/St_Vincent',
                            ),
                            ('America/Thule', '(GMT-0400) America/Thule'),
                            ('America/Tortola', '(GMT-0400) America/Tortola'),
                            (
                                'Atlantic/Bermuda',
                                '(GMT-0400) Atlantic/Bermuda',
                            ),
                            ('Canada/Atlantic', '(GMT-0400) Canada/Atlantic'),
                            (
                                'America/St_Johns',
                                '(GMT-0330) America/St_Johns',
                            ),
                            (
                                'Canada/Newfoundland',
                                '(GMT-0330) Canada/Newfoundland',
                            ),
                            (
                                'America/Araguaina',
                                '(GMT-0300) America/Araguaina',
                            ),
                            (
                                'America/Argentina/Buenos_Aires',
                                '(GMT-0300) America/Argentina/Buenos_Aires',
                            ),
                            (
                                'America/Argentina/Catamarca',
                                '(GMT-0300) America/Argentina/Catamarca',
                            ),
                            (
                                'America/Argentina/Cordoba',
                                '(GMT-0300) America/Argentina/Cordoba',
                            ),
                            (
                                'America/Argentina/Jujuy',
                                '(GMT-0300) America/Argentina/Jujuy',
                            ),
                            (
                                'America/Argentina/La_Rioja',
                                '(GMT-0300) America/Argentina/La_Rioja',
                            ),
                            (
                                'America/Argentina/Mendoza',
                                '(GMT-0300) America/Argentina/Mendoza',
                            ),
                            (
                                'America/Argentina/Rio_Gallegos',
                                '(GMT-0300) America/Argentina/Rio_Gallegos',
                            ),
                            (
                                'America/Argentina/Salta',
                                '(GMT-0300) America/Argentina/Salta',
                            ),
                            (
                                'America/Argentina/San_Juan',
                                '(GMT-0300) America/Argentina/San_Juan',
                            ),
                            (
                                'America/Argentina/San_Luis',
                                '(GMT-0300) America/Argentina/San_Luis',
                            ),
                            (
                                'America/Argentina/Tucuman',
                                '(GMT-0300) America/Argentina/Tucuman',
                            ),
                            (
                                'America/Argentina/Ushuaia',
                                '(GMT-0300) America/Argentina/Ushuaia',
                            ),
                            (
                                'America/Asuncion',
                                '(GMT-0300) America/Asuncion',
                            ),
                            ('America/Bahia', '(GMT-0300) America/Bahia'),
                            ('America/Belem', '(GMT-0300) America/Belem'),
                            ('America/Cayenne', '(GMT-0300) America/Cayenne'),
                            (
                                'America/Fortaleza',
                                '(GMT-0300) America/Fortaleza',
                            ),
                            ('America/Maceio', '(GMT-0300) America/Maceio'),
                            (
                                'America/Miquelon',
                                '(GMT-0300) America/Miquelon',
                            ),
                            (
                                'America/Montevideo',
                                '(GMT-0300) America/Montevideo',
                            ),
                            ('America/Nuuk', '(GMT-0300) America/Nuuk'),
                            (
                                'America/Paramaribo',
                                '(GMT-0300) America/Paramaribo',
                            ),
                            (
                                'America/Punta_Arenas',
                                '(GMT-0300) America/Punta_Arenas',
                            ),
                            ('America/Recife', '(GMT-0300) America/Recife'),
                            (
                                'America/Santarem',
                                '(GMT-0300) America/Santarem',
                            ),
                            (
                                'America/Santiago',
                                '(GMT-0300) America/Santiago',
                            ),
                            (
                                'America/Sao_Paulo',
                                '(GMT-0300) America/Sao_Paulo',
                            ),
                            (
                                'Antarctica/Palmer',
                                '(GMT-0300) Antarctica/Palmer',
                            ),
                            (
                                'Antarctica/Rothera',
                                '(GMT-0300) Antarctica/Rothera',
                            ),
                            (
                                'Atlantic/Stanley',
                                '(GMT-0300) Atlantic/Stanley',
                            ),
                            ('America/Noronha', '(GMT-0200) America/Noronha'),
                            (
                                'Atlantic/South_Georgia',
                                '(GMT-0200) Atlantic/South_Georgia',
                            ),
                            (
                                'America/Scoresbysund',
                                '(GMT-0100) America/Scoresbysund',
                            ),
                            ('Atlantic/Azores', '(GMT-0100) Atlantic/Azores'),
                            (
                                'Atlantic/Cape_Verde',
                                '(GMT-0100) Atlantic/Cape_Verde',
                            ),
                            ('Africa/Abidjan', '(GMT+0000) Africa/Abidjan'),
                            ('Africa/Accra', '(GMT+0000) Africa/Accra'),
                            ('Africa/Bamako', '(GMT+0000) Africa/Bamako'),
                            ('Africa/Banjul', '(GMT+0000) Africa/Banjul'),
                            ('Africa/Bissau', '(GMT+0000) Africa/Bissau'),
                            ('Africa/Conakry', '(GMT+0000) Africa/Conakry'),
                            ('Africa/Dakar', '(GMT+0000) Africa/Dakar'),
                            ('Africa/Freetown', '(GMT+0000) Africa/Freetown'),
                            ('Africa/Lome', '(GMT+0000) Africa/Lome'),
                            ('Africa/Monrovia', '(GMT+0000) Africa/Monrovia'),
                            (
                                'Africa/Nouakchott',
                                '(GMT+0000) Africa/Nouakchott',
                            ),
                            (
                                'Africa/Ouagadougou',
                                '(GMT+0000) Africa/Ouagadougou',
                            ),
                            ('Africa/Sao_Tome', '(GMT+0000) Africa/Sao_Tome'),
                            (
                                'America/Danmarkshavn',
                                '(GMT+0000) America/Danmarkshavn',
                            ),
                            (
                                'Antarctica/Troll',
                                '(GMT+0000) Antarctica/Troll',
                            ),
                            ('Atlantic/Canary', '(GMT+0000) Atlantic/Canary'),
                            ('Atlantic/Faroe', '(GMT+0000) Atlantic/Faroe'),
                            (
                                'Atlantic/Madeira',
                                '(GMT+0000) Atlantic/Madeira',
                            ),
                            (
                                'Atlantic/Reykjavik',
                                '(GMT+0000) Atlantic/Reykjavik',
                            ),
                            (
                                'Atlantic/St_Helena',
                                '(GMT+0000) Atlantic/St_Helena',
                            ),
                            ('Europe/Dublin', '(GMT+0000) Europe/Dublin'),
                            ('Europe/Guernsey', '(GMT+0000) Europe/Guernsey'),
                            (
                                'Europe/Isle_of_Man',
                                '(GMT+0000) Europe/Isle_of_Man',
                            ),
                            ('Europe/Jersey', '(GMT+0000) Europe/Jersey'),
                            ('Europe/Lisbon', '(GMT+0000) Europe/Lisbon'),
                            ('Europe/London', '(GMT+0000) Europe/London'),
                            ('GMT', '(GMT+0000) GMT'),
                            ('UTC', '(GMT+0000) UTC'),
                            ('Africa/Algiers', '(GMT+0100) Africa/Algiers'),
                            ('Africa/Bangui', '(GMT+0100) Africa/Bangui'),
                            (
                                'Africa/Brazzaville',
                                '(GMT+0100) Africa/Brazzaville',
                            ),
                            (
                                'Africa/Casablanca',
                                '(GMT+0100) Africa/Casablanca',
                            ),
                            ('Africa/Ceuta', '(GMT+0100) Africa/Ceuta'),
                            ('Africa/Douala', '(GMT+0100) Africa/Douala'),
                            ('Africa/El_Aaiun', '(GMT+0100) Africa/El_Aaiun'),
                            ('Africa/Kinshasa', '(GMT+0100) Africa/Kinshasa'),
                            ('Africa/Lagos', '(GMT+0100) Africa/Lagos'),
                            (
                                'Africa/Libreville',
                                '(GMT+0100) Africa/Libreville',
                            ),
                            ('Africa/Luanda', '(GMT+0100) Africa/Luanda'),
                            ('Africa/Malabo', '(GMT+0100) Africa/Malabo'),
                            ('Africa/Ndjamena', '(GMT+0100) Africa/Ndjamena'),
                            ('Africa/Niamey', '(GMT+0100) Africa/Niamey'),
                            (
                                'Africa/Porto-Novo',
                                '(GMT+0100) Africa/Porto-Novo',
                            ),
                            ('Africa/Tunis', '(GMT+0100) Africa/Tunis'),
                            (
                                'Arctic/Longyearbyen',
                                '(GMT+0100) Arctic/Longyearbyen',
                            ),
                            (
                                'Europe/Amsterdam',
                                '(GMT+0100) Europe/Amsterdam',
                            ),
                            ('Europe/Andorra', '(GMT+0100) Europe/Andorra'),
                            ('Europe/Belgrade', '(GMT+0100) Europe/Belgrade'),
                            ('Europe/Berlin', '(GMT+0100) Europe/Berlin'),
                            (
                                'Europe/Bratislava',
                                '(GMT+0100) Europe/Bratislava',
                            ),
                            ('Europe/Brussels', '(GMT+0100) Europe/Brussels'),
                            ('Europe/Budapest', '(GMT+0100) Europe/Budapest'),
                            ('Europe/Busingen', '(GMT+0100) Europe/Busingen'),
                            (
                                'Europe/Copenhagen',
                                '(GMT+0100) Europe/Copenhagen',
                            ),
                            (
                                'Europe/Gibraltar',
                                '(GMT+0100) Europe/Gibraltar',
                            ),
                            (
                                'Europe/Ljubljana',
                                '(GMT+0100) Europe/Ljubljana',
                            ),
                            (
                                'Europe/Luxembourg',
                                '(GMT+0100) Europe/Luxembourg',
                            ),
                            ('Europe/Madrid', '(GMT+0100) Europe/Madrid'),
                            ('Europe/Malta', '(GMT+0100) Europe/Malta'),
                            ('Europe/Monaco', '(GMT+0100) Europe/Monaco'),
                            ('Europe/Oslo', '(GMT+0100) Europe/Oslo'),
                            ('Europe/Paris', '(GMT+0100) Europe/Paris'),
                            (
                                'Europe/Podgorica',
                                '(GMT+0100) Europe/Podgorica',
                            ),
                            ('Europe/Prague', '(GMT+0100) Europe/Prague'),
                            ('Europe/Rome', '(GMT+0100) Europe/Rome'),
                            (
                                'Europe/San_Marino',
                                '(GMT+0100) Europe/San_Marino',
                            ),
                            ('Europe/Sarajevo', '(GMT+0100) Europe/Sarajevo'),
                            ('Europe/Skopje', '(GMT+0100) Europe/Skopje'),
                            (
                                'Europe/Stockholm',
                                '(GMT+0100) Europe/Stockholm',
                            ),
                            ('Europe/Tirane', '(GMT+0100) Europe/Tirane'),
                            ('Europe/Vaduz', '(GMT+0100) Europe/Vaduz'),
                            ('Europe/Vatican', '(GMT+0100) Europe/Vatican'),
                            ('Europe/Vienna', '(GMT+0100) Europe/Vienna'),
                            ('Europe/Warsaw', '(GMT+0100) Europe/Warsaw'),
                            ('Europe/Zagreb', '(GMT+0100) Europe/Zagreb'),
                            ('Europe/Zurich', '(GMT+0100) Europe/Zurich'),
                            ('Africa/Blantyre', '(GMT+0200) Africa/Blantyre'),
                            (
                                'Africa/Bujumbura',
                                '(GMT+0200) Africa/Bujumbura',
                            ),
                            ('Africa/Cairo', '(GMT+0200) Africa/Cairo'),
                            ('Africa/Gaborone', '(GMT+0200) Africa/Gaborone'),
                            ('Africa/Harare', '(GMT+0200) Africa/Harare'),
                            (
                                'Africa/Johannesburg',
                                '(GMT+0200) Africa/Johannesburg',
                            ),
                            ('Africa/Juba', '(GMT+0200) Africa/Juba'),
                            ('Africa/Khartoum', '(GMT+0200) Africa/Khartoum'),
                            ('Africa/Kigali', '(GMT+0200) Africa/Kigali'),
                            (
                                'Africa/Lubumbashi',
                                '(GMT+0200) Africa/Lubumbashi',
                            ),
                            ('Africa/Lusaka', '(GMT+0200) Africa/Lusaka'),
                            ('Africa/Maputo', '(GMT+0200) Africa/Maputo'),
                            ('Africa/Maseru', '(GMT+0200) Africa/Maseru'),
                            ('Africa/Mbabane', '(GMT+0200) Africa/Mbabane'),
                            ('Africa/Tripoli', '(GMT+0200) Africa/Tripoli'),
                            ('Africa/Windhoek', '(GMT+0200) Africa/Windhoek'),
                            ('Asia/Beirut', '(GMT+0200) Asia/Beirut'),
                            ('Asia/Famagusta', '(GMT+0200) Asia/Famagusta'),
                            ('Asia/Gaza', '(GMT+0200) Asia/Gaza'),
                            ('Asia/Hebron', '(GMT+0200) Asia/Hebron'),
                            ('Asia/Jerusalem', '(GMT+0200) Asia/Jerusalem'),
                            ('Asia/Nicosia', '(GMT+0200) Asia/Nicosia'),
                            ('Europe/Athens', '(GMT+0200) Europe/Athens'),
                            (
                                'Europe/Bucharest',
                                '(GMT+0200) Europe/Bucharest',
                            ),
                            ('Europe/Chisinau', '(GMT+0200) Europe/Chisinau'),
                            ('Europe/Helsinki', '(GMT+0200) Europe/Helsinki'),
                            (
                                'Europe/Kaliningrad',
                                '(GMT+0200) Europe/Kaliningrad',
                            ),
                            ('Europe/Kyiv', '(GMT+0200) Europe/Kyiv'),
                            (
                                'Europe/Mariehamn',
                                '(GMT+0200) Europe/Mariehamn',
                            ),
                            ('Europe/Riga', '(GMT+0200) Europe/Riga'),
                            ('Europe/Sofia', '(GMT+0200) Europe/Sofia'),
                            ('Europe/Tallinn', '(GMT+0200) Europe/Tallinn'),
                            ('Europe/Vilnius', '(GMT+0200) Europe/Vilnius'),
                            (
                                'Africa/Addis_Ababa',
                                '(GMT+0300) Africa/Addis_Ababa',
                            ),
                            ('Africa/Asmara', '(GMT+0300) Africa/Asmara'),
                            (
                                'Africa/Dar_es_Salaam',
                                '(GMT+0300) Africa/Dar_es_Salaam',
                            ),
                            ('Africa/Djibouti', '(GMT+0300) Africa/Djibouti'),
                            ('Africa/Kampala', '(GMT+0300) Africa/Kampala'),
                            (
                                'Africa/Mogadishu',
                                '(GMT+0300) Africa/Mogadishu',
                            ),
                            ('Africa/Nairobi', '(GMT+0300) Africa/Nairobi'),
                            (
                                'Antarctica/Syowa',
                                '(GMT+0300) Antarctica/Syowa',
                            ),
                            ('Asia/Aden', '(GMT+0300) Asia/Aden'),
                            ('Asia/Amman', '(GMT+0300) Asia/Amman'),
                            ('Asia/Baghdad', '(GMT+0300) Asia/Baghdad'),
                            ('Asia/Bahrain', '(GMT+0300) Asia/Bahrain'),
                            ('Asia/Damascus', '(GMT+0300) Asia/Damascus'),
                            ('Asia/Kuwait', '(GMT+0300) Asia/Kuwait'),
                            ('Asia/Qatar', '(GMT+0300) Asia/Qatar'),
                            ('Asia/Riyadh', '(GMT+0300) Asia/Riyadh'),
                            ('Europe/Istanbul', '(GMT+0300) Europe/Istanbul'),
                            ('Europe/Kirov', '(GMT+0300) Europe/Kirov'),
                            ('Europe/Minsk', '(GMT+0300) Europe/Minsk'),
                            ('Europe/Moscow', '(GMT+0300) Europe/Moscow'),
                            (
                                'Europe/Simferopol',
                                '(GMT+0300) Europe/Simferopol',
                            ),
                            (
                                'Europe/Volgograd',
                                '(GMT+0300) Europe/Volgograd',
                            ),
                            (
                                'Indian/Antananarivo',
                                '(GMT+0300) Indian/Antananarivo',
                            ),
                            ('Indian/Comoro', '(GMT+0300) Indian/Comoro'),
                            ('Indian/Mayotte', '(GMT+0300) Indian/Mayotte'),
                            ('Asia/Tehran', '(GMT+0330) Asia/Tehran'),
                            ('Asia/Baku', '(GMT+0400) Asia/Baku'),
                            ('Asia/Dubai', '(GMT+0400) Asia/Dubai'),
                            ('Asia/Muscat', '(GMT+0400) Asia/Muscat'),
                            ('Asia/Tbilisi', '(GMT+0400) Asia/Tbilisi'),
                            ('Asia/Yerevan', '(GMT+0400) Asia/Yerevan'),
                            (
                                'Europe/Astrakhan',
                                '(GMT+0400) Europe/Astrakhan',
                            ),
                            ('Europe/Samara', '(GMT+0400) Europe/Samara'),
                            ('Europe/Saratov', '(GMT+0400) Europe/Saratov'),
                            (
                                'Europe/Ulyanovsk',
                                '(GMT+0400) Europe/Ulyanovsk',
                            ),
                            ('Indian/Mahe', '(GMT+0400) Indian/Mahe'),
                            (
                                'Indian/Mauritius',
                                '(GMT+0400) Indian/Mauritius',
                            ),
                            ('Indian/Reunion', '(GMT+0400) Indian/Reunion'),
                            ('Asia/Kabul', '(GMT+0430) Asia/Kabul'),
                            (
                                'Antarctica/Mawson',
                                '(GMT+0500) Antarctica/Mawson',
                            ),
                            ('Asia/Aqtau', '(GMT+0500) Asia/Aqtau'),
                            ('Asia/Aqtobe', '(GMT+0500) Asia/Aqtobe'),
                            ('Asia/Ashgabat', '(GMT+0500) Asia/Ashgabat'),
                            ('Asia/Atyrau', '(GMT+0500) Asia/Atyrau'),
                            ('Asia/Dushanbe', '(GMT+0500) Asia/Dushanbe'),
                            ('Asia/Karachi', '(GMT+0500) Asia/Karachi'),
                            ('Asia/Oral', '(GMT+0500) Asia/Oral'),
                            ('Asia/Qyzylorda', '(GMT+0500) Asia/Qyzylorda'),
                            ('Asia/Samarkand', '(GMT+0500) Asia/Samarkand'),
                            ('Asia/Tashkent', '(GMT+0500) Asia/Tashkent'),
                            (
                                'Asia/Yekaterinburg',
                                '(GMT+0500) Asia/Yekaterinburg',
                            ),
                            (
                                'Indian/Kerguelen',
                                '(GMT+0500) Indian/Kerguelen',
                            ),
                            ('Indian/Maldives', '(GMT+0500) Indian/Maldives'),
                            ('Asia/Colombo', '(GMT+0530) Asia/Colombo'),
                            ('Asia/Kolkata', '(GMT+0530) Asia/Kolkata'),
                            ('Asia/Kathmandu', '(GMT+0545) Asia/Kathmandu'),
                            (
                                'Antarctica/Vostok',
                                '(GMT+0600) Antarctica/Vostok',
                            ),
                            ('Asia/Almaty', '(GMT+0600) Asia/Almaty'),
                            ('Asia/Bishkek', '(GMT+0600) Asia/Bishkek'),
                            ('Asia/Dhaka', '(GMT+0600) Asia/Dhaka'),
                            ('Asia/Omsk', '(GMT+0600) Asia/Omsk'),
                            ('Asia/Qostanay', '(GMT+0600) Asia/Qostanay'),
                            ('Asia/Thimphu', '(GMT+0600) Asia/Thimphu'),
                            ('Asia/Urumqi', '(GMT+0600) Asia/Urumqi'),
                            ('Indian/Chagos', '(GMT+0600) Indian/Chagos'),
                            ('Asia/Yangon', '(GMT+0630) Asia/Yangon'),
                            ('Indian/Cocos', '(GMT+0630) Indian/Cocos'),
                            (
                                'Antarctica/Davis',
                                '(GMT+0700) Antarctica/Davis',
                            ),
                            ('Asia/Bangkok', '(GMT+0700) Asia/Bangkok'),
                            ('Asia/Barnaul', '(GMT+0700) Asia/Barnaul'),
                            (
                                'Asia/Ho_Chi_Minh',
                                '(GMT+0700) Asia/Ho_Chi_Minh',
                            ),
                            ('Asia/Hovd', '(GMT+0700) Asia/Hovd'),
                            ('Asia/Jakarta', '(GMT+0700) Asia/Jakarta'),
                            (
                                'Asia/Krasnoyarsk',
                                '(GMT+0700) Asia/Krasnoyarsk',
                            ),
                            (
                                'Asia/Novokuznetsk',
                                '(GMT+0700) Asia/Novokuznetsk',
                            ),
                            (
                                'Asia/Novosibirsk',
                                '(GMT+0700) Asia/Novosibirsk',
                            ),
                            ('Asia/Phnom_Penh', '(GMT+0700) Asia/Phnom_Penh'),
                            ('Asia/Pontianak', '(GMT+0700) Asia/Pontianak'),
                            ('Asia/Tomsk', '(GMT+0700) Asia/Tomsk'),
                            ('Asia/Vientiane', '(GMT+0700) Asia/Vientiane'),
                            (
                                'Indian/Christmas',
                                '(GMT+0700) Indian/Christmas',
                            ),
                            ('Asia/Brunei', '(GMT+0800) Asia/Brunei'),
                            ('Asia/Choibalsan', '(GMT+0800) Asia/Choibalsan'),
                            ('Asia/Hong_Kong', '(GMT+0800) Asia/Hong_Kong'),
                            ('Asia/Irkutsk', '(GMT+0800) Asia/Irkutsk'),
                            (
                                'Asia/Kuala_Lumpur',
                                '(GMT+0800) Asia/Kuala_Lumpur',
                            ),
                            ('Asia/Kuching', '(GMT+0800) Asia/Kuching'),
                            ('Asia/Macau', '(GMT+0800) Asia/Macau'),
                            ('Asia/Makassar', '(GMT+0800) Asia/Makassar'),
                            ('Asia/Manila', '(GMT+0800) Asia/Manila'),
                            ('Asia/Shanghai', '(GMT+0800) Asia/Shanghai'),
                            ('Asia/Singapore', '(GMT+0800) Asia/Singapore'),
                            ('Asia/Taipei', '(GMT+0800) Asia/Taipei'),
                            (
                                'Asia/Ulaanbaatar',
                                '(GMT+0800) Asia/Ulaanbaatar',
                            ),
                            ('Australia/Perth', '(GMT+0800) Australia/Perth'),
                            ('Australia/Eucla', '(GMT+0845) Australia/Eucla'),
                            ('Asia/Chita', '(GMT+0900) Asia/Chita'),
                            ('Asia/Dili', '(GMT+0900) Asia/Dili'),
                            ('Asia/Jayapura', '(GMT+0900) Asia/Jayapura'),
                            ('Asia/Khandyga', '(GMT+0900) Asia/Khandyga'),
                            ('Asia/Pyongyang', '(GMT+0900) Asia/Pyongyang'),
                            ('Asia/Seoul', '(GMT+0900) Asia/Seoul'),
                            ('Asia/Tokyo', '(GMT+0900) Asia/Tokyo'),
                            ('Asia/Yakutsk', '(GMT+0900) Asia/Yakutsk'),
                            ('Pacific/Palau', '(GMT+0900) Pacific/Palau'),
                            (
                                'Australia/Darwin',
                                '(GMT+0930) Australia/Darwin',
                            ),
                            (
                                'Antarctica/DumontDUrville',
                                '(GMT+1000) Antarctica/DumontDUrville',
                            ),
                            ('Asia/Ust-Nera', '(GMT+1000) Asia/Ust-Nera'),
                            (
                                'Asia/Vladivostok',
                                '(GMT+1000) Asia/Vladivostok',
                            ),
                            (
                                'Australia/Brisbane',
                                '(GMT+1000) Australia/Brisbane',
                            ),
                            (
                                'Australia/Lindeman',
                                '(GMT+1000) Australia/Lindeman',
                            ),
                            ('Pacific/Chuuk', '(GMT+1000) Pacific/Chuuk'),
                            ('Pacific/Guam', '(GMT+1000) Pacific/Guam'),
                            (
                                'Pacific/Port_Moresby',
                                '(GMT+1000) Pacific/Port_Moresby',
                            ),
                            ('Pacific/Saipan', '(GMT+1000) Pacific/Saipan'),
                            (
                                'Australia/Adelaide',
                                '(GMT+1030) Australia/Adelaide',
                            ),
                            (
                                'Australia/Broken_Hill',
                                '(GMT+1030) Australia/Broken_Hill',
                            ),
                            (
                                'Antarctica/Casey',
                                '(GMT+1100) Antarctica/Casey',
                            ),
                            (
                                'Antarctica/Macquarie',
                                '(GMT+1100) Antarctica/Macquarie',
                            ),
                            ('Asia/Magadan', '(GMT+1100) Asia/Magadan'),
                            ('Asia/Sakhalin', '(GMT+1100) Asia/Sakhalin'),
                            (
                                'Asia/Srednekolymsk',
                                '(GMT+1100) Asia/Srednekolymsk',
                            ),
                            (
                                'Australia/Hobart',
                                '(GMT+1100) Australia/Hobart',
                            ),
                            (
                                'Australia/Lord_Howe',
                                '(GMT+1100) Australia/Lord_Howe',
                            ),
                            (
                                'Australia/Melbourne',
                                '(GMT+1100) Australia/Melbourne',
                            ),
                            (
                                'Australia/Sydney',
                                '(GMT+1100) Australia/Sydney',
                            ),
                            (
                                'Pacific/Bougainville',
                                '(GMT+1100) Pacific/Bougainville',
                            ),
                            ('Pacific/Efate', '(GMT+1100) Pacific/Efate'),
                            (
                                'Pacific/Guadalcanal',
                                '(GMT+1100) Pacific/Guadalcanal',
                            ),
                            ('Pacific/Kosrae', '(GMT+1100) Pacific/Kosrae'),
                            ('Pacific/Noumea', '(GMT+1100) Pacific/Noumea'),
                            ('Pacific/Pohnpei', '(GMT+1100) Pacific/Pohnpei'),
                            ('Asia/Anadyr', '(GMT+1200) Asia/Anadyr'),
                            ('Asia/Kamchatka', '(GMT+1200) Asia/Kamchatka'),
                            ('Pacific/Fiji', '(GMT+1200) Pacific/Fiji'),
                            (
                                'Pacific/Funafuti',
                                '(GMT+1200) Pacific/Funafuti',
                            ),
                            (
                                'Pacific/Kwajalein',
                                '(GMT+1200) Pacific/Kwajalein',
                            ),
                            ('Pacific/Majuro', '(GMT+1200) Pacific/Majuro'),
                            ('Pacific/Nauru', '(GMT+1200) Pacific/Nauru'),
                            ('Pacific/Norfolk', '(GMT+1200) Pacific/Norfolk'),
                            ('Pacific/Tarawa', '(GMT+1200) Pacific/Tarawa'),
                            ('Pacific/Wake', '(GMT+1200) Pacific/Wake'),
                            ('Pacific/Wallis', '(GMT+1200) Pacific/Wallis'),
                            (
                                'Antarctica/McMurdo',
                                '(GMT+1300) Antarctica/McMurdo',
                            ),
                            ('Pacific/Apia', '(GMT+1300) Pacific/Apia'),
                            (
                                'Pacific/Auckland',
                                '(GMT+1300) Pacific/Auckland',
                            ),
                            ('Pacific/Fakaofo', '(GMT+1300) Pacific/Fakaofo'),
                            ('Pacific/Kanton', '(GMT+1300) Pacific/Kanton'),
                            (
                                'Pacific/Tongatapu',
                                '(GMT+1300) Pacific/Tongatapu',
                            ),
                            ('Pacific/Chatham', '(GMT+1345) Pacific/Chatham'),
                            (
                                'Pacific/Kiritimati',
                                '(GMT+1400) Pacific/Kiritimati',
                            ),
                        ],
                        max_length=255,
                    ),
                ),
                (
                    'user',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='profile',
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



# Generated by Django 4.1.5 on 2023-02-03 19:50

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0011_chartrecord_user'),
    ]

    operations = [
        migrations.CreateModel(
            name='AudioScrobblerTSVImport',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                (
                    'tsv_file',
                    models.FileField(
                        blank=True,
                        null=True,
                        upload_to='audioscrobbler-uploads/%Y/%m-%d/',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-13 01:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('podcasts', '0004_episode_number'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='episode',
            options={},
        ),
        migrations.AlterField(
            model_name='episode',
            name='title',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-04 21:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('videos', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Scrobble',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('timestamp', models.DateTimeField(blank=True, null=True)),
                (
                    'playback_position_ticks',
                    models.PositiveIntegerField(blank=True, null=True),
                ),
                (
                    'playback_position',
                    models.CharField(blank=True, max_length=8, null=True),
                ),
                ('is_paused', models.BooleanField(default=False)),
                ('played_to_completion', models.BooleanField(default=False)),
                (
                    'source',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                ('source_id', models.TextField(blank=True, null=True)),
                (
                    'user',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    'video',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='videos.video',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



from rest_framework import serializers
from scrobbles.models import Scrobble, AudioScrobblerTSVImport


class AudioScrobblerTSVImportSerializer(serializers.ModelSerializer):
    class Meta:
        model = AudioScrobblerTSVImport
        fields = ('tsv_file',)


class ScrobbleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Scrobble
        fields = "__all__"



import csv
import logging
from datetime import datetime

import pytz
from music.models import Album, Artist, Track
from scrobbles.models import Scrobble

from vrobbler.apps.scrobbles.musicbrainz import (
    lookup_album_dict_from_mb,
    lookup_artist_id_from_mb,
)

logger = logging.getLogger(__name__)


def process_audioscrobbler_tsv_file(file_path, user_tz=None):
    """Takes a path to a file of TSV data and imports it as past scrobbles"""
    new_scrobbles = []
    if not user_tz:
        user_tz = pytz.utc

    with open(file_path) as infile:
        source = 'Audioscrobbler File'
        rows = csv.reader(infile, delimiter="\t")

        source_id = ""
        for row_num, row in enumerate(rows):
            if row_num in [0, 1, 2]:
                source_id += row[0] + "\n"
                continue
            if len(row) > 8:
                logger.warning(
                    'Improper row length during Audioscrobbler import',
                    extra={'row': row},
                )
                continue
            artist, artist_created = Artist.objects.get_or_create(name=row[0])
            if artist_created:
                artist.musicbrainz_id = lookup_artist_id_from_mb(artist.name)
                artist.save(update_fields=["musicbrainz_id"])

            album = None
            album_created = False
            albums = Album.objects.filter(name=row[1])
            if albums.count() == 1:
                album = albums.first()
            else:
                for potential_album in albums:
                    if artist in album.artist_set.all():
                        album = potential_album
            if not album:
                album_created = True
                album = Album.objects.create(name=row[1])
                album.save()
                album.artists.add(artist)

            if album_created:
                album_dict = lookup_album_dict_from_mb(
                    album.name, artist_name=artist.name
                )
                album.year = album_dict["year"]
                album.musicbrainz_id = album_dict["mb_id"]
                album.musicbrainz_releasegroup_id = album_dict["mb_group_id"]
                album.musicbrainz_albumartist_id = artist.musicbrainz_id
                album.save(
                    update_fields=[
                        "year",
                        "musicbrainz_id",
                        "musicbrainz_releasegroup_id",
                        "musicbrainz_albumartist_id",
                    ]
                )
                album.artists.add(artist)
                album.fetch_artwork()

            track, track_created = Track.objects.get_or_create(
                title=row[2],
                artist=artist,
                album=album,
            )

            if track_created:
                track.musicbrainz_id = row[7]
                track.run_time = int(row[4])
                track.run_time_ticks = int(row[4]) * 1000
                track.save()

            timestamp = (
                datetime.utcfromtimestamp(int(row[6]))
                .replace(tzinfo=user_tz)
                .astimezone(pytz.utc)
            )
            source = 'Audioscrobbler File'

            new_scrobble = Scrobble(
                timestamp=timestamp,
                source=source,
                source_id=source_id,
                track=track,
                played_to_completion=True,
                in_progress=False,
            )
            existing = Scrobble.objects.filter(
                timestamp=timestamp, track=track
            ).first()
            if existing:
                logger.debug(f"Skipping existing scrobble {new_scrobble}")
                continue
            logger.debug(f"Queued scrobble {new_scrobble} for creation")
            new_scrobbles.append(new_scrobble)

        created = Scrobble.objects.bulk_create(new_scrobbles)
        logger.info(
            f"Created {len(created)} scrobbles",
            extra={'created_scrobbles': created},
        )
        return created


def undo_audioscrobbler_tsv_import(process_log, dryrun=True):
    """Accepts the log from a TSV import and removes the scrobbles"""
    if not process_log:
        logger.warning("No lines in process log found to undo")
        return

    for line_num, line in enumerate(process_log.split('\n')):
        if line_num == 0:
            continue
        scrobble_id = line.split("\t")[0]
        scrobble = Scrobble.objects.filter(id=scrobble_id).first()
        if not scrobble:
            logger.warning(f"Could not find scrobble {scrobble_id} to undo")
            continue
        logger.info(f"Removing scrobble {scrobble_id}")
        if not dryrun:
            scrobble.delete()



# Generated by Django 4.1.5 on 2023-01-22 20:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0004_alter_league_options_alter_sport_options_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sportevent',
            name='season',
        ),
    ]



import json
import logging
from datetime import datetime

import pytz
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.db.models.fields import timezone
from django.http import FileResponse, HttpResponseRedirect, JsonResponse
from django.urls import reverse, reverse_lazy
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import FormView
from django.views.generic.edit import CreateView
from django.views.generic.list import ListView
from rest_framework import status
from rest_framework.decorators import (
    api_view,
    parser_classes,
    permission_classes,
)
from rest_framework.parsers import MultiPartParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from scrobbles.constants import (
    JELLYFIN_AUDIO_ITEM_TYPES,
    JELLYFIN_VIDEO_ITEM_TYPES,
)
from scrobbles.forms import ExportScrobbleForm, ScrobbleForm
from scrobbles.imdb import lookup_video_from_imdb
from scrobbles.models import AudioScrobblerTSVImport, Scrobble
from scrobbles.scrobblers import (
    jellyfin_scrobble_track,
    jellyfin_scrobble_video,
    manual_scrobble_event,
    manual_scrobble_video,
    mopidy_scrobble_podcast,
    mopidy_scrobble_track,
)
from scrobbles.serializers import (
    AudioScrobblerTSVImportSerializer,
    ScrobbleSerializer,
)
from scrobbles.thesportsdb import lookup_event_from_thesportsdb

from vrobbler.apps.music.aggregators import (
    scrobble_counts,
    top_artists,
    top_tracks,
    week_of_scrobbles,
)
from vrobbler.apps.scrobbles.export import export_scrobbles

logger = logging.getLogger(__name__)


class RecentScrobbleList(ListView):
    model = Scrobble

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        user = self.request.user
        now = timezone.now()
        if user.is_authenticated:
            if user.profile:
                timezone.activate(pytz.timezone(user.profile.timezone))
                now = timezone.localtime(timezone.now())
            data['now_playing_list'] = Scrobble.objects.filter(
                in_progress=True,
                is_paused=False,
                timestamp__lte=now,
                user=user,
            )

            completed_for_user = Scrobble.objects.filter(
                played_to_completion=True, user=user
            )
            data['video_scrobble_list'] = completed_for_user.filter(
                video__isnull=False
            ).order_by('-timestamp')[:15]

            data['podcast_scrobble_list'] = completed_for_user.filter(
                podcast_episode__isnull=False
            ).order_by('-timestamp')[:15]

            data['sport_scrobble_list'] = completed_for_user.filter(
                sport_event__isnull=False
            ).order_by('-timestamp')[:15]

            # data['top_daily_tracks'] = top_tracks()
            data['top_weekly_tracks'] = top_tracks(user, filter='week')
            data['top_monthly_tracks'] = top_tracks(user, filter='month')

            # data['top_daily_artists'] = top_artists()
            data['top_weekly_artists'] = top_artists(user, filter='week')
            data['top_monthly_artists'] = top_artists(user, filter='month')

        data["weekly_data"] = week_of_scrobbles(user=user)

        data['counts'] = scrobble_counts(user)
        data['imdb_form'] = ScrobbleForm
        data['export_form'] = ExportScrobbleForm
        return data

    def get_queryset(self):
        return Scrobble.objects.filter(
            track__isnull=False, in_progress=False
        ).order_by('-timestamp')[:15]


class ManualScrobbleView(FormView):
    form_class = ScrobbleForm
    template_name = 'scrobbles/manual_form.html'

    def form_valid(self, form):

        item_id = form.cleaned_data.get('item_id')
        data_dict = None
        if 'tt' in item_id:
            data_dict = lookup_video_from_imdb(
                form.cleaned_data.get('item_id')
            )
            if data_dict:
                manual_scrobble_video(data_dict, self.request.user.id)

        if not data_dict:
            logger.debug(f"Looking for sport event with ID {item_id}")
            data_dict = lookup_event_from_thesportsdb(
                form.cleaned_data.get('item_id')
            )
            if data_dict:
                manual_scrobble_event(data_dict, self.request.user.id)

        return HttpResponseRedirect(reverse("vrobbler-home"))


class JsonableResponseMixin:
    """
    Mixin to add JSON support to a form.
    Must be used with an object-based FormView (e.g. CreateView)
    """

    def form_invalid(self, form):
        response = super().form_invalid(form)
        if self.request.accepts('text/html'):
            return response
        else:
            return JsonResponse(form.errors, status=400)

    def form_valid(self, form):
        # We make sure to call the parent's form_valid() method because
        # it might do some processing (in the case of CreateView, it will
        # call form.save() for example).
        response = super().form_valid(form)
        if self.request.accepts('text/html'):
            return response
        else:
            data = {
                'pk': self.object.pk,
            }
            return JsonResponse(data)


class AudioScrobblerImportCreateView(
    LoginRequiredMixin, JsonableResponseMixin, CreateView
):
    model = AudioScrobblerTSVImport
    fields = ['tsv_file']
    template_name = 'scrobbles/upload_form.html'
    success_url = reverse_lazy('vrobbler-home')

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        return HttpResponseRedirect(self.get_success_url())


@csrf_exempt
@api_view(['GET'])
def scrobble_endpoint(request):
    """List all Scrobbles, or create a new Scrobble"""
    scrobble = Scrobble.objects.all()
    serializer = ScrobbleSerializer(scrobble, many=True)
    return Response(serializer.data)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
def jellyfin_websocket(request):
    data_dict = request.data

    if (
        data_dict['NotificationType'] == 'PlaybackProgress'
        and data_dict['ItemType'] == 'Audio'
    ):
        return Response({}, status=status.HTTP_304_NOT_MODIFIED)

    # For making things easier to build new input processors
    if getattr(settings, "DUMP_REQUEST_DATA", False):
        json_data = json.dumps(data_dict, indent=4)
        logger.debug(f"{json_data}")

    scrobble = None
    media_type = data_dict.get("ItemType", "")

    if media_type in JELLYFIN_AUDIO_ITEM_TYPES:
        scrobble = jellyfin_scrobble_track(data_dict, request.user.id)

    if media_type in JELLYFIN_VIDEO_ITEM_TYPES:
        scrobble = jellyfin_scrobble_video(data_dict, request.user.id)

    if not scrobble:
        return Response({}, status=status.HTTP_400_BAD_REQUEST)

    return Response({'scrobble_id': scrobble.id}, status=status.HTTP_200_OK)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
def mopidy_websocket(request):
    try:
        data_dict = json.loads(request.data)
    except TypeError:
        logger.warning('Received Mopidy data as dict, rather than a string')
        data_dict = request.data

    # For making things easier to build new input processors
    if getattr(settings, "DUMP_REQUEST_DATA", False):
        json_data = json.dumps(data_dict, indent=4)
        logger.debug(f"{json_data}")

    if 'podcast' in data_dict.get('mopidy_uri'):
        scrobble = mopidy_scrobble_podcast(data_dict, request.user.id)
    else:
        scrobble = mopidy_scrobble_track(data_dict, request.user.id)

    if not scrobble:
        return Response({}, status=status.HTTP_400_BAD_REQUEST)

    return Response({'scrobble_id': scrobble.id}, status=status.HTTP_200_OK)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
@parser_classes([MultiPartParser])
def import_audioscrobbler_file(request):
    """Takes a TSV file in the Audioscrobbler format, saves it and processes the
    scrobbles.
    """
    scrobbles_created = []
    # tsv_file = request.FILES[0]

    file_serializer = AudioScrobblerTSVImportSerializer(data=request.data)
    if file_serializer.is_valid():
        import_file = file_serializer.save()
        return Response(
            {'scrobble_ids': scrobbles_created}, status=status.HTTP_200_OK
        )
    else:
        return Response(
            file_serializer.errors, status=status.HTTP_400_BAD_REQUEST
        )


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['GET'])
def scrobble_finish(request, uuid):
    user = request.user
    if not user.is_authenticated:
        return Response({}, status=status.HTTP_403_FORBIDDEN)

    scrobble = Scrobble.objects.filter(user=user, uuid=uuid).first()
    if not scrobble:
        return Response({}, status=status.HTTP_404_NOT_FOUND)
    scrobble.stop(force_finish=True)
    return Response(
        {'id': scrobble.id, 'status': scrobble.status},
        status=status.HTTP_200_OK,
    )


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['GET'])
def scrobble_cancel(request, uuid):
    user = request.user
    if not user.is_authenticated:
        return Response({}, status=status.HTTP_403_FORBIDDEN)

    scrobble = Scrobble.objects.filter(user=user, uuid=uuid).first()
    if not scrobble:
        return Response({}, status=status.HTTP_404_NOT_FOUND)
    scrobble.cancel()
    return Response(
        {'id': scrobble.id, 'status': 'cancelled'}, status=status.HTTP_200_OK
    )


@permission_classes([IsAuthenticated])
@api_view(['GET'])
def export(request):
    format = request.GET.get('export_type', 'csv')
    start = request.GET.get('start')
    end = request.GET.get('end')
    logger.debug(f"Exporting all scrobbles in format {format}")

    temp_file, extension = export_scrobbles(
        start_date=start, end_date=end, format=format
    )

    now = datetime.now()
    filename = f"vrobbler-export-{str(now)}.{extension}"
    response = FileResponse(open(temp_file, 'rb'))
    response["Content-Disposition"] = f'attachment; filename="{filename}"'

    return response



import pytz

from django.contrib.auth import get_user_model
from django.db import models
from django_extensions.db.models import TimeStampedModel
from profiles.constants import PRETTY_TIMEZONE_CHOICES

User = get_user_model()


class UserProfile(TimeStampedModel):
    user = models.OneToOneField(
        User, on_delete=models.CASCADE, related_name="profile"
    )
    timezone = models.CharField(
        max_length=255, choices=PRETTY_TIMEZONE_CHOICES
    )

    def __str__(self):
        return f"User profile for {self.user}"

    @property
    def tzinfo(self):
        return pytz.timezone(self.timezone)



# Generated by Django 4.1.5 on 2023-01-12 17:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('podcasts', '0002_episode_run_time_episode_run_time_ticks_and_more'),
        ('scrobbles', '0006_scrobble_track_alter_scrobble_video'),
    ]

    operations = [
        migrations.AddField(
            model_name='scrobble',
            name='podcast_episode',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to='podcasts.episode',
            ),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-12 17:18

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Producer',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Podcast',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
                ('active', models.BooleanField(default=True)),
                ('url', models.URLField(blank=True, null=True)),
                (
                    'producer',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='podcasts.producer',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Episode',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('title', models.CharField(max_length=255)),
                (
                    'uuid',
                    models.UUIDField(
                        blank=True,
                        default=uuid.uuid4,
                        editable=False,
                        null=True,
                    ),
                ),
                (
                    'mopidy_uri',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                (
                    'podcast',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='podcasts.producer',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-12 17:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music', '0006_album_artists'),
    ]

    operations = [
        migrations.AlterField(
            model_name='album',
            name='artists',
            field=models.ManyToManyField(to='music.artist'),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-05 16:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('videos', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='series',
            options={'verbose_name_plural': 'series'},
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-04 21:33

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Series',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                ('overview', models.TextField(blank=True, null=True)),
                ('tagline', models.TextField(blank=True, null=True)),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                (
                    'video_type',
                    models.CharField(
                        choices=[
                            ('U', 'Unknown'),
                            ('E', 'TV Episode'),
                            ('M', 'Movie'),
                        ],
                        default='U',
                        max_length=1,
                    ),
                ),
                (
                    'title',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                ('overview', models.TextField(blank=True, null=True)),
                ('tagline', models.TextField(blank=True, null=True)),
                (
                    'run_time',
                    models.CharField(blank=True, max_length=8, null=True),
                ),
                (
                    'run_time_ticks',
                    models.BigIntegerField(blank=True, null=True),
                ),
                ('year', models.IntegerField()),
                ('season_number', models.IntegerField(blank=True, null=True)),
                ('episode_number', models.IntegerField(blank=True, null=True)),
                (
                    'tvdb_id',
                    models.CharField(blank=True, max_length=20, null=True),
                ),
                (
                    'imdb_id',
                    models.CharField(blank=True, max_length=20, null=True),
                ),
                (
                    'tvrage_id',
                    models.CharField(blank=True, max_length=20, null=True),
                ),
                (
                    'tv_series',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to='videos.series',
                    ),
                ),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



from django import forms


class ExportScrobbleForm(forms.Form):
    """Provide options for downloading scrobbles"""

    EXPORT_TYPES = (
        ('as', 'Audioscrobbler'),
        ('csv', 'CSV'),
        ('html', 'HTML'),
    )
    export_type = forms.ChoiceField(choices=EXPORT_TYPES)


class ScrobbleForm(forms.Form):
    item_id = forms.CharField(
        label="",
        widget=forms.TextInput(
            attrs={
                'class': "form-control form-control-dark w-100",
                'placeholder': "Scrobble something (IMDB ID, String, TVDB ID ...)",
                'aria-label': "Scrobble something",
            }
        ),
    )



import logging

from dateutil.parser import parse
from django.conf import settings
from django.utils import timezone
from pysportsdb import TheSportsDbClient
from sports.models import Sport

logger = logging.getLogger(__name__)

API_KEY = getattr(settings, "THESPORTSDB_API_KEY", "2")
client = TheSportsDbClient(api_key=API_KEY)


def lookup_event_from_thesportsdb(event_id: str) -> dict:

    event = client.lookup_event(event_id)['events'][0]
    if not event or type(event) != dict:
        return {}
    league = {}  # client.lookup_league(league_id=event.get('idLeague'))
    event_type = "Game"
    sport, _created = Sport.objects.get_or_create(
        thesportsdb_id=event.get('strSport')
    )

    data_dict = {
        "ItemType": sport.default_event_type,
        "Name": event.get('strEvent'),
        "AltName": event.get('strEventAlternate'),
        "Start": parse(event.get('strTimestamp')),
        "Provider_thesportsdb": event.get('idEvent'),
        "RunTime": sport.default_event_run_time,
        "RunTimeTicks": sport.default_event_run_time_ticks,
        "Sport": event.get('strSport'),
        "Season": event.get('strSeason'),
        "LeagueId": event.get('idLeague'),
        "LeagueName": event.get('strLeague'),
        "HomeTeamId": event.get('idHomeTeam'),
        "HomeTeamName": event.get('strHomeTeam'),
        "AwayTeamId": event.get('idAwayTeam'),
        "AwayTeamName": event.get('strAwayTeam'),
        "RoundId": event.get('intRound'),
        "PlaybackPositionTicks": None,
        "PlaybackPosition": None,
        "UtcTimestamp": timezone.now().strftime('%Y-%m-%d %H:%M:%S.%f%z'),
        "IsPaused": False,
        "PlayedToCompletion": False,
        "Source": "Vrobbler",
    }

    return data_dict



import logging
from typing import Dict
from uuid import uuid4

from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from sqlalchemy import update
from scrobbles.mixins import ScrobblableMixin
from vrobbler.apps.sports.utils import (
    get_players_from_event,
    get_round_name_from_event,
)

logger = logging.getLogger(__name__)
BNULL = {"blank": True, "null": True}


class SportEventType(models.TextChoices):
    UNKNOWN = 'UK', _('Unknown')
    GAME = 'GA', _('Game')
    MATCH = 'MA', _('Match')


class TheSportsDbMixin(TimeStampedModel):
    name = models.CharField(max_length=255)
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)
    thesportsdb_id = models.CharField(max_length=255, **BNULL)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class Sport(TheSportsDbMixin):
    default_event_run_time = models.IntegerField(**BNULL)
    default_event_type = models.CharField(
        max_length=2,
        choices=SportEventType.choices,
        default=SportEventType.UNKNOWN,
    )

    # TODO Add these to the default run_time for Football
    # run_time_seconds = 11700
    # run_time_ticks = run_time_seconds * 1000
    @property
    def default_event_run_time_ticks(self):
        default_run_time = getattr(
            settings, 'DEFAULT_EVENT_RUNTIME_SECONDS', 14400
        )
        if self.default_event_run_time:
            default_run_time = self.default_event_run_time
        return default_run_time * 1000


class League(TheSportsDbMixin):
    logo = models.ImageField(upload_to="sports/league-logos/", **BNULL)
    abbreviation_str = models.CharField(max_length=10, **BNULL)
    sport = models.ForeignKey(Sport, on_delete=models.DO_NOTHING, **BNULL)

    @property
    def abbreviation(self):
        return self.abbreviation_str


class Season(TheSportsDbMixin):
    league = models.ForeignKey(League, on_delete=models.DO_NOTHING, **BNULL)

    def __str__(self):
        return f'{self.name} season of {self.league}'


class Team(TheSportsDbMixin):
    league = models.ForeignKey(League, on_delete=models.DO_NOTHING, **BNULL)


class Player(TheSportsDbMixin):
    league = models.ForeignKey(League, on_delete=models.DO_NOTHING, **BNULL)
    team = models.ForeignKey(Team, on_delete=models.DO_NOTHING, **BNULL)


class Round(TheSportsDbMixin):
    season = models.ForeignKey(Season, on_delete=models.DO_NOTHING, **BNULL)

    def __str__(self):
        return f'{self.name} of {self.season}'


class SportEvent(ScrobblableMixin):
    COMPLETION_PERCENT = getattr(settings, 'SPORT_COMPLETION_PERCENT', 90)

    event_type = models.CharField(
        max_length=2,
        choices=SportEventType.choices,
        default=SportEventType.UNKNOWN,
    )
    round = models.ForeignKey(Round, on_delete=models.DO_NOTHING, **BNULL)
    start = models.DateTimeField(**BNULL)
    home_team = models.ForeignKey(
        Team,
        on_delete=models.DO_NOTHING,
        related_name='home_event_set',
        **BNULL,
    )
    away_team = models.ForeignKey(
        Team,
        on_delete=models.DO_NOTHING,
        related_name='away_event_set',
        **BNULL,
    )
    player_one = models.ForeignKey(
        Player,
        on_delete=models.DO_NOTHING,
        related_name='player_one_set',
        **BNULL,
    )
    player_two = models.ForeignKey(
        Player,
        on_delete=models.DO_NOTHING,
        related_name='player_two_set',
        **BNULL,
    )

    def __str__(self):
        return f"{self.start.date()} - {self.round} - {self.home_team} v {self.away_team}"

    def get_absolute_url(self):
        return reverse("sports:event_detail", kwargs={'slug': self.uuid})

    @classmethod
    def find_or_create(cls, data_dict: Dict) -> "Event":
        """Given a data dict from Jellyfin, does the heavy lifting of looking up
        the video and, if need, TV Series, creating both if they don't yet
        exist.

        """
        # Find or create our Sport
        sid = data_dict.get("Sport")
        sport, s_created = Sport.objects.get_or_create(thesportsdb_id=sid)
        if s_created:
            sport.name = sid
            sport.save(update_fields=['name'])

        # Find or create our League
        lid = data_dict.get("LeagueId")
        league, l_created = League.objects.get_or_create(
            thesportsdb_id=lid, sport=sport
        )
        if l_created:
            league.sport = sport
            league.name = data_dict.get("LeagueName", "")
            league.save(update_fields=['sport', 'name'])

        # Find or create our Season
        seid = data_dict.get('Season')
        season, se_created = Season.objects.get_or_create(
            thesportsdb_id=seid, league=league
        )
        if se_created:
            season.name = seid
            season.save(update_fields=['name'])

        # Find or create our Round
        rid = data_dict.get('RoundId')
        round, r_created = Round.objects.get_or_create(
            thesportsdb_id=rid, season=season
        )
        if r_created:
            round.season = season
            round.save(update_fields=['season'])

        # Set some special data for Tennis
        player_one = None
        player_two = None
        if data_dict.get('Sport') == 'Tennis':
            event_name = data_dict.get('Name', '')
            if not round.name:
                round.name = get_round_name_from_event(event_name)
                round.save(update_fields=['name'])

            players_list = get_players_from_event(event_name)
            player_one = Player.objects.filter(
                name__icontains=players_list[0]
            ).first()
            if not player_one:
                player_one = Player.objects.create(name=players_list[0])
            player_two = Player.objects.filter(
                name__icontains=players_list[1]
            ).first()
            if not player_two:
                player_two = Player.objects.create(name=players_list[1])

        home_team = None
        away_team = None
        if data_dict.get("HomeTeamName"):
            home_team_dict = {
                "name": data_dict.get("HomeTeamName", ""),
                "thesportsdb_id": data_dict.get("HomeTeamId", ""),
                "league": league,
            }
            home_team, _created = Team.objects.get_or_create(**home_team_dict)

            away_team_dict = {
                "name": data_dict.get("AwayTeamName", ""),
                "thesportsdb_id": data_dict.get("AwayTeamId", ""),
                "league": league,
            }
            away_team, _created = Team.objects.get_or_create(**away_team_dict)

        event_dict = {
            "title": data_dict.get("Name"),
            "event_type": sport.default_event_type,
            "home_team": home_team,
            "away_team": away_team,
            "player_one": player_one,
            "player_two": player_two,
            "start": data_dict['Start'],
            "round": round,
            "run_time_ticks": data_dict.get("RunTimeTicks"),
            "run_time": data_dict.get("RunTime", ""),
        }
        event, _created = cls.objects.get_or_create(**event_dict)

        return event



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vrobbler',
 'vrobbler.apps.music',
 'vrobbler.apps.music.migrations',
 'vrobbler.apps.podcasts',
 'vrobbler.apps.podcasts.migrations',
 'vrobbler.apps.profiles',
 'vrobbler.apps.profiles.migrations',
 'vrobbler.apps.scrobbles',
 'vrobbler.apps.scrobbles.migrations',
 'vrobbler.apps.sports',
 'vrobbler.apps.sports.migrations',
 'vrobbler.apps.videogames',
 'vrobbler.apps.videos',
 'vrobbler.apps.videos.migrations']

package_data = \
{'': ['*'],
 'vrobbler': ['static/images/*',
              'templates/*',
              'templates/music/*',
              'templates/scrobbles/*',
              'templates/videos/*']}

install_requires = \
['Django>=4.0.3,<5.0.0',
 'Markdown>=3.3.6,<4.0.0',
 'Pillow>=9.0.1,<10.0.0',
 'cinemagoer>=2022.12.27,<2023.0.0',
 'colorlog>=6.6.0,<7.0.0',
 'dj-database-url>=0.5.0,<0.6.0',
 'django-allauth>=0.50.0,<0.51.0',
 'django-cachalot>=2.5.2,<3.0.0',
 'django-celery-results>=2.3.0,<3.0.0',
 'django-extensions>=3.1.5,<4.0.0',
 'django-filter>=21.1,<22.0',
 'django-markdownify>=0.9.1,<0.10.0',
 'django-mathfilters>=1.0.0,<2.0.0',
 'django-redis>=5.2.0,<6.0.0',
 'django-simple-history>=3.1.1,<4.0.0',
 'django-taggit>=2.1.0,<3.0.0',
 'djangorestframework>=3.13.1,<4.0.0',
 'gunicorn>=20.1.0,<21.0.0',
 'musicbrainzngs>=0.7.1,<0.8.0',
 'psycopg2>=2.9.3,<3.0.0',
 'pysportsdb>=0.1.0,<0.2.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'python-json-logger>=2.0.2,<3.0.0',
 'pytz>=2022.7.1,<2023.0.0',
 'redis>=4.2.2,<5.0.0',
 'whitenoise>=6.3.0,<7.0.0']

entry_points = \
{'console_scripts': ['vrobbler = vrobbler.cli:main']}

setup_kwargs = {
    'name': 'vrobbler',
    'version': '0.8.6',
    'description': '',
    'long_description': 'None',
    'author': 'Colin Powell',
    'author_email': 'colin@unbl.ink',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 4.1.5 on 2023-01-13 01:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('music', '0007_alter_album_artists'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='track',
            options={},
        ),
    ]



from django.urls import path
from videos import views

app_name = 'videos'


urlpatterns = [
    # path('', views.scrobble_endpoint, name='scrobble-list'),
    path("movies/", views.MovieListView.as_view(), name='movie_list'),
    path('series/', views.SeriesListView.as_view(), name='series_list'),
    path(
        'series/<slug:slug>/',
        views.SeriesDetailView.as_view(),
        name='series_detail',
    ),
    path(
        'video/<slug:slug>/',
        views.VideoDetailView.as_view(),
        name='video_detail',
    ),
]



import datetime

import pytz
from django.conf import settings
from django.utils import timezone


# need to translate to a non-naive timezone, even if timezone == settings.TIME_ZONE, so we can compare two dates
def to_user_timezone(date, profile):
    timezone = profile.timezone if profile.timezone else settings.TIME_ZONE
    return date.replace(tzinfo=pytz.timezone(settings.TIME_ZONE)).astimezone(
        pytz.timezone(timezone)
    )


def to_system_timezone(date, profile):
    timezone = profile.timezone if profile.timezone else settings.TIME_ZONE
    return date.replace(tzinfo=pytz.timezone(timezone)).astimezone(
        pytz.timezone(settings.TIME_ZONE)
    )


def now_user_timezone(profile):
    timezone.activate(pytz.timezone(profile.timezone))
    return timezone.localtime(timezone.now())


def now_system_timezone():
    return (
        datetime.datetime.now()
        .replace(tzinfo=pytz.timezone(settings.TIME_ZONE))
        .astimezone(pytz.timezone(settings.TIME_ZONE))
    )



import logging
from typing import Dict, Optional
from uuid import uuid4

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel

from vrobbler.apps.scrobbles.mixins import ScrobblableMixin

logger = logging.getLogger(__name__)
BNULL = {"blank": True, "null": True}


class Producer(TimeStampedModel):
    name = models.CharField(max_length=255)
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)

    def __str__(self):
        return f"{self.name}"


class Podcast(TimeStampedModel):
    name = models.CharField(max_length=255)
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)
    producer = models.ForeignKey(
        Producer, on_delete=models.DO_NOTHING, **BNULL
    )
    active = models.BooleanField(default=True)
    url = models.URLField(**BNULL)

    def __str__(self):
        return f"{self.name}"


class Episode(ScrobblableMixin):
    COMPLETION_PERCENT = getattr(settings, 'PODCAST_COMPLETION_PERCENT', 90)

    podcast = models.ForeignKey(Podcast, on_delete=models.DO_NOTHING)
    number = models.IntegerField(**BNULL)
    pub_date = models.DateField(**BNULL)
    mopidy_uri = models.CharField(max_length=255, **BNULL)

    def __str__(self):
        return f"{self.title}"

    @classmethod
    def find_or_create(
        cls, podcast_dict: Dict, producer_dict: Dict, episode_dict: Dict
    ) -> Optional["Episode"]:
        """Given a data dict from Mopidy, finds or creates a podcast and
        producer before saving the epsiode so it can be scrobbled.

        """
        if not podcast_dict.get('name'):
            logger.warning(f"No name from source for podcast, not scrobbling")
            return

        producer = None
        if producer_dict.get('name'):
            producer, producer_created = Producer.objects.get_or_create(
                **producer_dict
            )
            if producer_created:
                logger.debug(f"Created new producer {producer}")
            else:
                logger.debug(f"Found producer {producer}")

        if producer:
            podcast_dict["producer_id"] = producer.id
        podcast, podcast_created = Podcast.objects.get_or_create(
            **podcast_dict
        )
        if podcast_created:
            logger.debug(f"Created new podcast {podcast}")
        else:
            logger.debug(f"Found podcast {podcast}")

        episode_dict['podcast_id'] = podcast.id

        episode, created = cls.objects.get_or_create(**episode_dict)
        if created:
            logger.debug(f"Created new episode: {episode}")
        else:
            logger.debug(f"Found episode {episode}")

        return episode



from datetime import datetime, timedelta
from typing import List, Optional

import pytz
from django.db.models import Count, Q, Sum
from django.utils import timezone
from music.models import Artist, Track
from scrobbles.models import Scrobble
from videos.models import Video
from vrobbler.apps.profiles.utils import now_user_timezone

NOW = timezone.now()
START_OF_TODAY = datetime.combine(NOW.date(), datetime.min.time(), NOW.tzinfo)
STARTING_DAY_OF_CURRENT_WEEK = NOW.date() - timedelta(
    days=NOW.today().isoweekday() % 7
)
STARTING_DAY_OF_CURRENT_MONTH = NOW.date().replace(day=1)
STARTING_DAY_OF_CURRENT_YEAR = NOW.date().replace(month=1, day=1)


def scrobble_counts(user=None):

    now = timezone.now()
    user_filter = Q()
    if user and user.is_authenticated:
        now = now_user_timezone(user.profile)
        user_filter = Q(user=user)

    start_of_today = datetime.combine(
        now.date(), datetime.min.time(), now.tzinfo
    )
    starting_day_of_current_week = now.date() - timedelta(
        days=now.today().isoweekday() % 7
    )
    starting_day_of_current_month = now.date().replace(day=1)
    starting_day_of_current_year = now.date().replace(month=1, day=1)

    finished_scrobbles_qs = Scrobble.objects.filter(
        user_filter, played_to_completion=True
    )
    data = {}
    data['today'] = finished_scrobbles_qs.filter(
        timestamp__gte=start_of_today
    ).count()
    data['week'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_week
    ).count()
    data['month'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_month
    ).count()
    data['year'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_year
    ).count()
    data['alltime'] = finished_scrobbles_qs.count()
    return data


def week_of_scrobbles(user=None, media: str = 'tracks') -> dict[str, int]:

    now = timezone.now()
    user_filter = Q()
    if user and user.is_authenticated:
        now = now_user_timezone(user.profile)
        user_filter = Q(user=user)

    start_of_today = datetime.combine(
        now.date(), datetime.min.time(), now.tzinfo
    )

    scrobble_day_dict = {}
    base_qs = Scrobble.objects.filter(user_filter, played_to_completion=True)

    media_filter = Q(track__isnull=False)
    if media == 'movies':
        media_filter = Q(video__video_type=Video.VideoType.MOVIE)
    if media == 'series':
        media_filter = Q(video__video_type=Video.VideoType.TV_EPISODE)

    for day in range(6, -1, -1):
        start = start_of_today - timedelta(days=day)
        end = datetime.combine(start, datetime.max.time(), now.tzinfo)
        day_of_week = start.strftime('%A')

        scrobble_day_dict[day_of_week] = base_qs.filter(
            media_filter,
            timestamp__gte=start,
            timestamp__lte=end,
            played_to_completion=True,
        ).count()

    return scrobble_day_dict


def top_tracks(
    user: "User", filter: str = "today", limit: int = 15
) -> List["Track"]:

    now = timezone.now()
    if user.is_authenticated:
        now = now_user_timezone(user.profile)

    start_of_today = datetime.combine(
        now.date(), datetime.min.time(), now.tzinfo
    )
    starting_day_of_current_week = now.date() - timedelta(
        days=now.today().isoweekday() % 7
    )
    starting_day_of_current_month = now.date().replace(day=1)
    starting_day_of_current_year = now.date().replace(month=1, day=1)

    time_filter = Q(scrobble__timestamp__gte=start_of_today)
    if filter == "week":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_week)
    if filter == "month":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_month)
    if filter == "year":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_year)

    return (
        Track.objects.filter(time_filter)
        .annotate(num_scrobbles=Count("scrobble", distinct=True))
        .order_by("-num_scrobbles")[:limit]
    )


def top_artists(
    user: "User", filter: str = "today", limit: int = 15
) -> List["Artist"]:
    time_filter = Q(track__scrobble__timestamp__gte=START_OF_TODAY)
    if filter == "week":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_WEEK
        )
    if filter == "month":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_MONTH
        )
    if filter == "year":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_YEAR
        )

    return (
        Artist.objects.filter(time_filter)
        .annotate(num_scrobbles=Count("track__scrobble", distinct=True))
        .order_by("-num_scrobbles")[:limit]
    )


def artist_scrobble_count(artist_id: int, filter: str = "today") -> int:
    return Scrobble.objects.filter(track__artist=artist_id).count()



import logging
from typing import Dict
from uuid import uuid4

from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from scrobbles.mixins import ScrobblableMixin

logger = logging.getLogger(__name__)
BNULL = {"blank": True, "null": True}


class VideoGameCollection(TimeStampedModel):
    name = models.CharField(max_length=255)
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)
    cover = models.ImageField(upload_to="games/series-covers/", **BNULL)
    igdb_id = models.IntegerField(**BNULL)

    def __str__(self):
        return self.name


class VideoGame(ScrobblableMixin):
    COMPLETION_PERCENT = getattr(settings, 'GAME_COMPLETION_PERCENT', 100)

    collection = models.ForeignKey(VideoGameCollection, **BNULL)
    igdb_id = models.IntegerField(**BNULL)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("videogames:game_detail", kwargs={'slug': self.uuid})

    @classmethod
    def find_or_create(cls, data_dict: Dict) -> "Event":
        """Given a data dict from Jellyfin, does the heavy lifting of looking up
        the video and, if need, TV Series, creating both if they don't yet
        exist.

        """
        league_dict = {
            "abbreviation_str": data_dict.get("LeagueName", ""),
            "thesportsdb_id": data_dict.get("LeagueId", ""),
        }
        league, _created = League.objects.get_or_create(**league_dict)

        home_team_dict = {
            "name": data_dict.get("HomeTeamName", ""),
            "thesportsdb_id": data_dict.get("HomeTeamId", ""),
            "league": league,
        }
        home_team, _created = Team.objects.get_or_create(**home_team_dict)

        away_team_dict = {
            "name": data_dict.get("AwayTeamName", ""),
            "thesportsdb_id": data_dict.get("AwayTeamId", ""),
            "league": league,
        }
        away_team, _created = Team.objects.get_or_create(**away_team_dict)

        event_dict = {
            "title": data_dict.get("Name"),
            "event_type": data_dict.get("ItemType"),
            "home_team": home_team,
            "away_team": away_team,
            "start": data_dict['Start'],
            "league": league,
            "run_time_ticks": data_dict.get("RunTimeTicks"),
            "run_time": data_dict.get("RunTime", ""),
        }
        event, _created = cls.objects.get_or_create(**event_dict)

        return event


#!/usr/bin/env python3
